@@ -26,6 +26,7 @@ end
 
 function Behaviour:SetUnit(unit)
 	self.unit = unit
+	self.engineID = unit.engineID
 end
 
 
@@ -48,3 +49,11 @@ end
 function Behaviour:Passive()
 	return false
 end
+
+function Behaviour:UnitMoveFailed(unit)
+	self:UnitIdle(unit)
+end
+
+function Behaviour:OwnerDied()
+	return
+end
\ No newline at end of file
