@@ -1,13 +1,15 @@
 
-require "taskqueues"
-require "taskqueuebehaviour"
-require "attackerbehaviour"
+shard_include(  "taskqueues" )
+shard_include(  "taskqueuebehaviour" )
+shard_include(  "attackerbehaviour" )
+shard_include(  "bootbehaviour" )
 
 behaviours = { }
 
 function defaultBehaviours(unit)
 	b = {}
 	u = unit:Internal()
+	table.insert(b, BootBehaviour )
 	if u:CanBuild() then
 		table.insert(b,TaskQueueBehaviour)
 	else
