@@ -10,14 +10,14 @@ public:
 	typedef boost::shared_ptr<CSpringDamage> Ptr;
 
 	CSpringDamage(CSpringGame* game, springai::OOAICallback* callback, SUnitDamagedEvent* evt);
-	virtual ~CSpringDamage();
+	virtual ~CSpringDamage() override;
 
-	virtual float Damage();
-	virtual Position Direction();
-	virtual std::string DamageType();
-	virtual std::string WeaponType();
-	virtual IUnit* Attacker();
-	virtual std::vector<std::string> Effects();
+	virtual float Damage() override;
+	virtual Position Direction() override;
+	virtual std::string DamageType() override;
+	virtual std::string WeaponType() override;
+	virtual IUnit* Attacker() override;
+	virtual std::vector<std::string> Effects() override;
 
 
 protected:
