@@ -26,22 +26,15 @@ fi
 mkdir -p installer/downloads
 cd installer/downloads
 
-$WGET http://www.springlobby.info/windows/latest.zip
 if ! [ -s vcredist_x86.exe ]; then
 	$WGET http://download.microsoft.com/download/e/1/c/e1c773de-73ba-494a-a5ba-f24906ecf088/vcredist_x86.exe
 fi
 
 if [ ! -s spring_testing_minimal-portable.7z ]; then
 	echo "Warning: spring_testing_minimal-portable.7z didn't exist, downloading..." >&2
-	$WGET http://springrts.com/dl/buildbot/default/master/spring_testing_minimal-portable.7z
+	$WGET https://springrts.com/dl/buildbot/default/master/spring_testing_minimal-portable.7z
 fi
 
-cd ..
-rm -rf Springlobby
-mkdir -p Springlobby
-cd Springlobby
-unzip ../downloads/latest.zip -d SLArchive
-
 cd ../..
 
 #create uninstall.nsh
