@@ -121,7 +121,7 @@ float3 CAICheats::GetUnitPos(int unitId) const
 {
 	const CUnit* unit = GetUnit(unitId);
 	if (unit) {
-		return unit->pos;
+		return unit->midPos;
 	}
 
 	return ZeroVector;
