@@ -15,45 +15,6 @@
 
 CInMapDrawModel* inMapDrawerModel = NULL;
 
-CR_BIND(CInMapDrawModel, )
-CR_REG_METADATA(CInMapDrawModel, (
-	CR_MEMBER(drawQuadsX),
-	CR_MEMBER(drawQuadsY),
-	CR_MEMBER(drawQuads),
-	CR_IGNORED(drawAllMarks),
-	CR_MEMBER(numPoints),
-	CR_MEMBER(numLines),
-	CR_POSTLOAD(PostLoad)
-))
-
-CR_BIND(CInMapDrawModel::MapDrawPrimitive, (false, -1, NULL))
-
-CR_REG_METADATA_SUB(CInMapDrawModel, MapDrawPrimitive, (
-	CR_MEMBER(spectator),
-	CR_MEMBER(teamID),
-	CR_MEMBER(teamController)
-))
-
-CR_BIND_DERIVED(CInMapDrawModel::MapPoint, CInMapDrawModel::MapDrawPrimitive, (false, -1, NULL, ZeroVector, ""))
-CR_REG_METADATA_SUB(CInMapDrawModel, MapPoint, (
-	CR_MEMBER(pos),
-	CR_MEMBER(label)
-))
-
-CR_BIND_DERIVED(CInMapDrawModel::MapLine, CInMapDrawModel::MapDrawPrimitive, (false, -1, NULL, ZeroVector, ZeroVector))
-CR_REG_METADATA_SUB(CInMapDrawModel, MapLine, (
-	CR_MEMBER(pos1),
-	CR_MEMBER(pos2)
-))
-
-CR_BIND(CInMapDrawModel::DrawQuad, )
-CR_REG_METADATA_SUB(CInMapDrawModel, DrawQuad, /*(
-	CR_MEMBER(points),
-	CR_MEMBER(lines)
-)*/)
-
-
-
 const size_t CInMapDrawModel::DRAW_QUAD_SIZE = 32;
 
 const float CInMapDrawModel::QUAD_SCALE = 1.0f / (DRAW_QUAD_SIZE * SQUARE_SIZE);
@@ -75,14 +36,6 @@ CInMapDrawModel::~CInMapDrawModel()
 {
 }
 
-void CInMapDrawModel::PostLoad()
-{
-	if (drawQuads.size() != (drawQuadsX * drawQuadsY)) {
-		// For old savegames
-		drawQuads.resize(drawQuadsX * drawQuadsY);
-	}
-}
-
 bool CInMapDrawModel::MapDrawPrimitive::IsLocalPlayerAllowedToSee(const CInMapDrawModel* inMapDrawModel) const
 {
 	const int allyTeam = teamHandler->AllyTeam(teamID);
