@@ -10,12 +10,17 @@
 #include "Sim/Misc/CollisionHandler.h"
 #include "Sim/Misc/CollisionVolume.h"
 #include "Sim/Misc/GeometricObjects.h"
+#include "Sim/Misc/LosHandler.h"
 #include "Sim/Misc/QuadField.h"
 #include "Sim/Misc/TeamHandler.h"
-#include "Sim/Misc/RadarHandler.h"
+#include "Sim/Units/UnitHandler.h"
 #include "Sim/Units/UnitTypes/Factory.h"
+#include "Sim/Weapons/PlasmaRepulser.h"
+#include "Sim/Weapons/WeaponDef.h"
 #include "System/myMath.h"
 
+#include <algorithm>
+#include <vector>
 
 //////////////////////////////////////////////////////////////////////
 // Local/Helper functions
@@ -32,9 +37,9 @@ inline static bool TestConeHelper(
 	const float spread,
 	const CSolidObject* obj)
 {
-	const CollisionVolume* cv = obj->collisionVolume;
+	const CollisionVolume* cv = &obj->collisionVolume;
 
-	const float3 objVec3D = cv->GetWorldSpacePos(obj, ZeroVector) - pos3D;
+	const float3 objVec3D = cv->GetWorldSpacePos(obj) - pos3D;
 	const float  objDst1D = Clamp(objVec3D.dot(dir3D), 0.0f, length);
 	const float  coneSize = math::fabs(objDst1D) * spread + 1.0f;
 
@@ -45,9 +50,11 @@ inline static bool TestConeHelper(
 	bool ret = false;
 
 	if (obj->GetBlockingMapID() < unitHandler->MaxUnits()) {
+		// obj is a unit
 		if (!ret) { ret = ((cv->GetPointSurfaceDistance(static_cast<const CUnit*>(obj), NULL,    pos3D) - coneSize) <= 0.0f); }
 		if (!ret) { ret = ((cv->GetPointSurfaceDistance(static_cast<const CUnit*>(obj), NULL, expPos3D) - coneSize) <= 0.0f); }
 	} else {
+		// obj is a feature
 		if (!ret) { ret = ((cv->GetPointSurfaceDistance(static_cast<const CFeature*>(obj), NULL,    pos3D) - coneSize) <= 0.0f); }
 		if (!ret) { ret = ((cv->GetPointSurfaceDistance(static_cast<const CFeature*>(obj), NULL, expPos3D) - coneSize) <= 0.0f); }
 	}
@@ -103,9 +110,9 @@ inline static bool TestTrajectoryConeHelper(
 	//   THE TRAJECTORY CURVE MIGHT STILL INTERSECT
 	//   EVEN WHEN <x, f(x)> DOES NOT LIE INSIDE CV
 	//   SO THIS CAN GENERATE FALSE NEGATIVES
-	const CollisionVolume* cv = obj->collisionVolume;
+	const CollisionVolume* cv = &obj->collisionVolume;
 
-	const float3 objVec3D = cv->GetWorldSpacePos(obj, ZeroVector) - pos3D;
+	const float3 objVec3D = cv->GetWorldSpacePos(obj) - pos3D;
 	const float  objDst1D = Clamp(objVec3D.dot(dir2D), 0.0f, length);
 	const float  coneSize = math::fabs(objDst1D) * spread + baseSize;
 
@@ -203,7 +210,7 @@ float TraceRay(
 					if (!f->HasCollidableStateBit(CSolidObject::CSTATE_BIT_QUADMAPRAYS))
 						continue;
 
-					if (CCollisionHandler::DetectHit(f, start, start + dir * length, &cq, true)) {
+					if (CCollisionHandler::DetectHit(f, f->GetTransformMatrix(true), start, start + dir * length, &cq, true)) {
 						const float len = cq.GetHitPosDist(start, dir);
 
 						// we want the closest feature (intersection point) on the ray
@@ -236,7 +243,7 @@ float TraceRay(
 					if (ignoreCloaked && u->IsCloaked())
 						continue;
 
-					if (CCollisionHandler::DetectHit(u, start, start + dir * length, &cq, true)) {
+					if (CCollisionHandler::DetectHit(u, u->GetTransformMatrix(true), start, start + dir * length, &cq, true)) {
 						const float len = cq.GetHitPosDist(start, dir);
 
 						// we want the closest unit (intersection point) on the ray
@@ -270,27 +277,62 @@ float TraceRay(
 }
 
 
+void TraceRayShields(
+	const CWeapon* emitter,
+	const float3& start,
+	const float3& dir,
+	float length,
+	std::vector<SShieldDist>& hitShields
+) {
+	CollisionQuery cq;
+
+	const auto& quads = quadField->GetQuadsOnRay(start, dir, length);
+	for (const int quadIdx: quads) {
+		const CQuadField::Quad& quad = quadField->GetQuad(quadIdx);
+
+		for (CPlasmaRepulser* r: quad.repulsers) {
+			if (!r->CanIntercept(emitter->weaponDef->interceptedByShieldType, emitter->owner->allyteam))
+				continue;
+
+			if (CCollisionHandler::DetectHit(r->owner, &r->collisionVolume, r->owner->GetTransformMatrix(true), start, start + dir * length, &cq, true)) {
+				if (cq.InsideHit() && r->weaponDef->exteriorShield)
+					continue;
+
+				const float len = cq.GetHitPosDist(start, dir);
+				if (len <= 0.0f)
+					continue;
+
+				const auto hitCmp = [](const float a, const SShieldDist& b) { return (a < b.dist); };
+				const auto insPos = std::upper_bound(hitShields.begin(), hitShields.end(), len, hitCmp);
+
+				hitShields.insert(insPos, {r, len});
+			}
+		}
+	}
+}
+
+
 float GuiTraceRay(
 	const float3& start,
 	const float3& dir,
 	const float length,
 	const CUnit* exclude,
-	CUnit*& hitUnit,
-	CFeature*& hitFeature,
+	const CUnit*& hitUnit,
+	const CFeature*& hitFeature,
 	bool useRadar,
 	bool groundOnly,
 	bool ignoreWater
 ) {
-	hitUnit = NULL;
-	hitFeature = NULL;
+	hitUnit = nullptr;
+	hitFeature = nullptr;
 
 	if (dir == ZeroVector)
 		return -1.0f;
 
 	// ground and water-plane intersection
-	const float guiRayLength = length;
-	const float groundRayLength = CGround::LineGroundCol(start, start + dir * guiRayLength, false);
-	const float waterRayLength = math::floorf(math::fabs(start.y / std::min(dir.y, -0.00001f)));
+	const float    guiRayLength = length;
+	const float groundRayLength = CGround::LineGroundCol(start, dir, guiRayLength, false);
+	const float  waterRayLength = CGround::LinePlaneCol(start, dir, guiRayLength, 0.0f);
 
 	float minRayLength = groundRayLength;
 	float minIngressDist = length;
@@ -312,55 +354,55 @@ float GuiTraceRay(
 		const CQuadField::Quad& quad = quadField->GetQuad(quadIdx);
 
 		// Unit Intersection
-		for (CUnit* unit: quad.units) {
-			const bool unitIsEnemy = !teamHandler->Ally(unit->allyteam, gu->myAllyTeam);
-			const bool unitOnRadar = (useRadar && radarHandler->InRadar(unit, gu->myAllyTeam));
-			const bool unitInSight = (unit->losStatus[gu->myAllyTeam] & (LOS_INLOS | LOS_CONTRADAR));
+		for (const CUnit* u: quad.units) {
+			const bool unitIsEnemy = !teamHandler->Ally(u->allyteam, gu->myAllyTeam);
+			const bool unitOnRadar = (useRadar && losHandler->InRadar(u, gu->myAllyTeam));
+			const bool unitInSight = (u->losStatus[gu->myAllyTeam] & (LOS_INLOS | LOS_CONTRADAR));
 			const bool unitVisible = !unitIsEnemy || unitOnRadar || unitInSight || gu->spectatingFullView;
 
-			if (unit == exclude)
+			if (u == exclude)
 				continue;
 			// test this bit only in synced traces, rely on noSelect here
-			if (false && !unit->HasCollidableStateBit(CSolidObject::CSTATE_BIT_QUADMAPRAYS))
+			if (false && !u->HasCollidableStateBit(CSolidObject::CSTATE_BIT_QUADMAPRAYS))
 				continue;
-			if (unit->noSelect)
+			if (u->noSelect)
 				continue;
 			if (!unitVisible)
 				continue;
 
-			CollisionVolume cv(unit->collisionVolume);
+			CollisionVolume cv = u->collisionVolume;
 
-			if (unit->isIcon || (!unitInSight && unitOnRadar && unitIsEnemy)) {
+			if (u->isIcon || (!unitInSight && unitOnRadar && unitIsEnemy)) {
 				// for iconified units, just pretend the collision
 				// volume is a sphere of radius <unit->IconRadius>
 				// (count radar blips as such too)
-				cv.InitSphere(unit->iconRadius);
+				cv.InitSphere(u->iconRadius);
 			}
 
-			if (CCollisionHandler::MouseHit(unit, start, start + dir * guiRayLength, &cv, &cq)) {
+			if (CCollisionHandler::MouseHit(u, u->GetTransformMatrix(false), start, start + dir * guiRayLength, &cv, &cq)) {
 				// get the distance to the ray-volume ingress point
 				// (not likely to generate inside-hit special cases)
 				const float ingressDist = cq.GetIngressPosDist(start, dir);
 				const float  egressDist = cq.GetEgressPosDist(start, dir);
 
-				const bool factoryUnderCursor = unit->unitDef->IsFactoryUnit();
+				const bool factoryUnderCursor = u->unitDef->IsFactoryUnit();
 				const bool factoryHitBeforeUnit = ((hitFactory && ingressDist < minIngressDist) || (!hitFactory &&  egressDist < minIngressDist));
 				const bool unitHitInsideFactory = ((hitFactory && ingressDist <  minEgressDist) || (!hitFactory && ingressDist < minIngressDist));
 
 				// give units in a factory higher priority than the factory itself
-				if (hitUnit == NULL || (factoryUnderCursor && factoryHitBeforeUnit) || (!factoryUnderCursor && unitHitInsideFactory)) {
+				if (hitUnit == nullptr || (factoryUnderCursor && factoryHitBeforeUnit) || (!factoryUnderCursor && unitHitInsideFactory)) {
 					hitFactory = factoryUnderCursor;
 					minIngressDist = ingressDist;
 					minEgressDist = egressDist;
 
-					hitUnit = unit;
-					hitFeature = NULL;
+					hitUnit = u;
+					hitFeature = nullptr;
 				}
 			}
 		}
 
 		// Feature Intersection
-		for (CFeature* f: quad.features) {
+		for (const CFeature* f: quad.features) {
 			if (!gu->spectatingFullView && !f->IsInLosForAllyTeam(gu->myAllyTeam))
 				continue;
 			// test this bit only in synced traces, rely on noSelect here
@@ -369,7 +411,9 @@ float GuiTraceRay(
 			if (f->noSelect)
 				continue;
 
-			if (CCollisionHandler::DetectHit(f, start, start + dir * guiRayLength, &cq, true)) {
+			CollisionVolume cv = f->collisionVolume;
+
+			if (CCollisionHandler::MouseHit(f, f->GetTransformMatrix(false), start, start + dir * guiRayLength, &cv, &cq)) {
 				const float hitDist = cq.GetHitPosDist(start, dir);
 
 				const bool factoryHitBeforeUnit = ( hitFactory && hitDist <  minEgressDist);
@@ -377,12 +421,12 @@ float GuiTraceRay(
 
 				// we want the closest feature (intersection point) on the ray
 				// give features in a factory (?) higher priority than the factory itself
-				if (hitUnit == NULL || factoryHitBeforeUnit || unitHitInsideFactory) {
+				if (hitUnit == nullptr || factoryHitBeforeUnit || unitHitInsideFactory) {
 					hitFactory = false;
 					minIngressDist = hitDist;
 
 					hitFeature = f;
-					hitUnit = NULL;
+					hitUnit = nullptr;
 				}
 			}
 		}
@@ -391,8 +435,8 @@ float GuiTraceRay(
 	if ((minRayLength > 0.0f) && ((minRayLength + 200.0f) < minIngressDist)) {
 		minIngressDist = minRayLength;
 
-		hitUnit    = NULL;
-		hitFeature = NULL;
+		hitUnit    = nullptr;
+		hitFeature = nullptr;
 	}
 
 	return minIngressDist;
