@@ -26,9 +26,7 @@ std::string CKeyCodes::GetName(int code) const
 {
 	const auto it = codeToName.find(code);
 	if (it == codeToName.end()) {
-		char buf[64];
-		SNPRINTF(buf, sizeof(buf), "0x%03X", code);
-		return buf;
+		return IntToString(code, "0x%03X");
 	}
 	return it->second;
 }
@@ -38,9 +36,7 @@ std::string CKeyCodes::GetDefaultName(int code) const
 {
 	const auto it = defaultCodeToName.find(code);
 	if (it == defaultCodeToName.end()) {
-		char buf[64];
-		SNPRINTF(buf, sizeof(buf), "0x%03X", code);
-		return buf;
+		return IntToString(code, "0x%03X");
 	}
 	return it->second;
 }
@@ -75,8 +71,7 @@ bool CKeyCodes::IsValidLabel(const std::string& label)
 	if (!isalpha(label[0])) {
 		return false;
 	}
-	for (int i = 0; i < (int)label.size(); i++) {
-		const char c = label[i];
+	for (const char& c: label) {
 		if (!isalnum(c) && (c != '_')) {
 			return false;
 		}
@@ -277,11 +272,6 @@ void CKeyCodes::SaveUserKeySymbols(FILE* file) const
 			// this keysym is not standard
 			const int code = p.second;
 			std::string name = GetDefaultName(code);
-			if (name.empty()) {
-				char buf[16];
-				SNPRINTF(buf, 16, "0x%03X", code);
-				name = buf;
-			}
 			fprintf(file, "keysym  %-10s  %s\n", keysym.c_str(), name.c_str());
 			output = true;
 		}
