@@ -12,14 +12,13 @@ struct lua_State;
 class CLuaIntro : public CLuaHandle
 {
 	public:
-		static void LoadHandler();
-		static void FreeHandler();
+		static bool ReloadHandler() { return (FreeHandler(), LoadFreeHandler()); } // NOTE the ','
+		static bool LoadFreeHandler() { return (LoadHandler() || FreeHandler()); }
+
+		static bool LoadHandler();
+		static bool FreeHandler();
 
 	public: // call-ins
-		bool HasCallIn(lua_State *L, const std::string& name);
-		bool UnsyncedUpdateCallIn(lua_State *L, const std::string& name);
-
-		void Shutdown();
 		void DrawLoadScreen();
 		void LoadProgress(const std::string& msg, const bool replace_lastline);
 
@@ -30,10 +29,10 @@ class CLuaIntro : public CLuaHandle
 		std::string LoadFile(const std::string& filename) const;
 
 	private:
-		static bool LoadUnsyncedCtrlFunctions(lua_State *L);
-		static bool LoadUnsyncedReadFunctions(lua_State *L);
-		static bool LoadSyncedReadFunctions(lua_State *L);
-		static bool RemoveSomeOpenGLFunctions(lua_State *L);
+		static bool LoadUnsyncedCtrlFunctions(lua_State* L);
+		static bool LoadUnsyncedReadFunctions(lua_State* L);
+		static bool LoadSyncedReadFunctions(lua_State* L);
+		static bool RemoveSomeOpenGLFunctions(lua_State* L);
 };
 
 
