@@ -168,10 +168,12 @@ public:
 	const float* GetCornerHeightMapSynced() const { return sharedCornerHeightMaps[true]; }
 	const float3* GetFaceNormalsSynced()    const { return sharedFaceNormals[true]; }
 	const float3* GetCenterNormalsSynced()  const { return sharedCenterNormals[true]; }
+	const float3* GetCenterNormals2DSynced()  const { return sharedCenterNormals2D[true]; }
 	/// unsynced versions
 	const float* GetCornerHeightMapUnsynced() const { return sharedCornerHeightMaps[false]; }
 	const float3* GetFaceNormalsUnsynced()    const { return sharedFaceNormals[false]; }
 	const float3* GetCenterNormalsUnsynced()  const { return sharedCenterNormals[false]; }
+	const float3* GetCenterNormals2DUnsynced()  const { return sharedCenterNormals2D[false]; }
 
 
 	/// shared interface
@@ -179,6 +181,7 @@ public:
 	const float* GetSharedCenterHeightMap(bool synced) const { return sharedCenterHeightMaps[synced]; }
 	const float3* GetSharedFaceNormals(bool synced) const { return sharedFaceNormals[synced]; }
 	const float3* GetSharedCenterNormals(bool synced) const { return sharedCenterNormals[synced]; }
+	const float3* GetSharedCenterNormals2D(bool synced) const { return sharedCenterNormals2D[synced]; }
 	const float* GetSharedSlopeMap(bool synced) const { return sharedSlopeMaps[synced]; }
 
 	/// if you modify the heightmap through these, call UpdateHeightMapSynced
@@ -186,14 +189,14 @@ public:
 	float AddHeight(const int idx, const float a);
 
 
-	float GetInitMinHeight() const { return initMinHeight; }
-	float GetCurrMinHeight() const { return currMinHeight; }
-	float GetInitMaxHeight() const { return initMaxHeight; }
-	float GetCurrMaxHeight() const { return currMaxHeight; }
+	float GetInitMinHeight() const { return initHeightBounds.x; }
+	float GetCurrMinHeight() const { return currHeightBounds.x; }
+	float GetInitMaxHeight() const { return initHeightBounds.y; }
+	float GetCurrMaxHeight() const { return currHeightBounds.y; }
 	float GetBoundingRadius() const { return boundingRadius; }
 
-	bool IsUnderWater() const { return (currMaxHeight <  0.0f); }
-	bool IsAboveWater() const { return (currMinHeight >= 0.0f); }
+	bool IsUnderWater() const { return (currHeightBounds.y <  0.0f); }
+	bool IsAboveWater() const { return (currHeightBounds.x >= 0.0f); }
 
 	bool HasVisibleWater() const;
 	bool HasOnlyVoidWater() const;
@@ -241,6 +244,8 @@ protected:
 	std::vector<float3> faceNormalsUnsynced;   //< size: 2*mapx      *  mapy     , contains 2 normals per quad -> triangle strip [UNSYNCED]
 	std::vector<float3> centerNormalsSynced;   //< size:   mapx      *  mapy     , contains 1 interpolated normal per quad, same as (facenormal0+facenormal1).Normalize()) [SYNCED]
 	std::vector<float3> centerNormalsUnsynced;
+	std::vector<float3> centerNormals2DSynced;
+	std::vector<float3> centerNormals2DUnsynced;
 
 	std::vector<float> slopeMap;               //< size: (mapx/2)    * (mapy/2)  , same as 1.0 - interpolate(centernomal[i]).y [SYNCED]
 	std::vector<unsigned char> typeMap;
@@ -255,6 +260,7 @@ private:
 	const float* sharedCenterHeightMaps[2];
 	const float3* sharedFaceNormals[2];
 	const float3* sharedCenterNormals[2];
+	const float3* sharedCenterNormals2D[2];
 	const float* sharedSlopeMaps[2];
 
 #ifdef USE_UNSYNCED_HEIGHTMAP
@@ -266,8 +272,8 @@ private:
 
 	unsigned int mapChecksum;
 
-	float initMinHeight, initMaxHeight; //< initial minimum- and maximum-height (before any deformations)
-	float currMinHeight, currMaxHeight; //< current minimum- and maximum-height
+	float2 initHeightBounds; //< initial minimum- and maximum-height (before any deformations)
+	float2 currHeightBounds; //< current minimum- and maximum-height
 	float boundingRadius;
 };
 
@@ -284,8 +290,8 @@ inline float CReadMap::SetHeight(const int idx, const float h, const int add) {
 	// add=1 <--> x = x*1 + h = x+h
 	x = x * add + h;
 
-	currMinHeight = std::min(x, currMinHeight);
-	currMaxHeight = std::max(x, currMaxHeight);
+	currHeightBounds.x = std::min(x, currHeightBounds.x);
+	currHeightBounds.y = std::max(x, currHeightBounds.y);
 
 	return x;
 }
