@@ -28,59 +28,63 @@
 // -------------------------------------------------------------------------------------------------
 // STATICS
 
+
+static int MAX_POOL_SIZE = 8000000;
+
 Patch::RenderMode Patch::renderMode = Patch::VBO;
 
 // one pool per thread
 static size_t poolSize = 0;
-static std::vector<CTriNodePool*> pools;
+static std::vector<CTriNodePool> pools[CRoamMeshDrawer::MESH_COUNT];
 
-void CTriNodePool::InitPools(const size_t newPoolSize)
+void CTriNodePool::InitPools(bool shadowPass, size_t newPoolSize)
 {
-	if (!pools.empty())
+	if (!pools[shadowPass].empty())
 		return;
 
 	// int numThreads = GetNumThreads();
 	int numThreads = ThreadPool::GetMaxThreads();
-	const size_t allocPerThread = std::max(newPoolSize / numThreads, newPoolSize / 3);
+	const size_t allocPerThread = std::max(newPoolSize / numThreads, newPoolSize / 3) & (~(size_t)0x1);
 
-	poolSize = newPoolSize;
-
-	pools.reserve(numThreads);
-
-	for (; numThreads > 0; --numThreads) {
-		pools.push_back(new CTriNodePool(allocPerThread));
+	try {
+		poolSize = newPoolSize;
+		pools[shadowPass].reserve(numThreads);
+		for (; numThreads > 0; --numThreads) {
+			pools[shadowPass].emplace_back(allocPerThread);
+		}
+	} catch(const std::bad_alloc& e) {
+		LOG_L(L_FATAL, "Failed to allocate memory for ROAM (reducing pool size): %s", e.what());
+		MAX_POOL_SIZE = newPoolSize * 0.75f;
+		FreePools(shadowPass);
+		InitPools(shadowPass, MAX_POOL_SIZE);
 	}
 }
 
-void CTriNodePool::FreePools()
+void CTriNodePool::FreePools(bool shadowPass)
 {
-	for (CTriNodePool* pool: pools) {
-		delete pool;
-	}
-
-	pools.clear();
+	pools[shadowPass].clear();
 }
 
 
-void CTriNodePool::ResetAll()
+void CTriNodePool::ResetAll(bool shadowPass)
 {
 	bool outOfNodes = false;
 
-	for (CTriNodePool* pool: pools) {
-		outOfNodes |= pool->OutOfNodes();
-		pool->Reset();
+	for (CTriNodePool& pool: pools[shadowPass]) {
+		outOfNodes |= pool.OutOfNodes();
+		pool.Reset();
 	}
 
 	if (outOfNodes && (poolSize < MAX_POOL_SIZE)) {
-		FreePools();
-		InitPools(std::min(poolSize * 2, size_t(MAX_POOL_SIZE)));
+		FreePools(shadowPass);
+		InitPools(shadowPass, std::min<size_t>(poolSize * 2, MAX_POOL_SIZE));
 	}
 }
 
 
-CTriNodePool* CTriNodePool::GetPool()
+CTriNodePool* CTriNodePool::GetPool(bool shadowPass)
 {
-	return (pools[ThreadPool::GetThreadNum()]);
+	return &(pools[shadowPass][ThreadPool::GetThreadNum()]);
 }
 
 
@@ -88,6 +92,14 @@ CTriNodePool* CTriNodePool::GetPool()
 // -------------------------------------------------------------------------------------------------
 // CTriNodePool Class
 
+CTriNodePool::CTriNodePool(const size_t poolSize)
+{
+	assert((poolSize & 0x1) == 0); // we always allocate left & right, so we need an even pool
+	m_NextTriNode = 0;
+	pool.resize(poolSize);
+}
+
+
 void CTriNodePool::Reset()
 {
 	// reinit all entries to NULL
@@ -99,13 +111,14 @@ void CTriNodePool::Reset()
 }
 
 
-TriTreeNode* CTriNodePool::AllocateTri()
+void CTriNodePool::Allocate(TriTreeNode*& left, TriTreeNode*& right)
 {
 	// IF we've run out of TriTreeNodes, just return NULL (this is handled gracefully)
 	if (OutOfNodes())
-		return nullptr;
+		return;
 
-	return &pool[m_NextTriNode++];
+	left  = &(pool[m_NextTriNode++]);
+	right = &(pool[m_NextTriNode++]);
 }
 
 
@@ -116,8 +129,8 @@ TriTreeNode* CTriNodePool::AllocateTri()
 
 Patch::Patch()
 	: smfGroundDrawer(nullptr)
-	, heightMap(nullptr)
 	, currentVariance(nullptr)
+	, currentPool(nullptr)
 	, isDirty(true)
 	, vboVerticesUploaded(false)
 	, varianceMaxLimit(std::numeric_limits<float>::max())
@@ -158,7 +171,7 @@ void Patch::Init(CSMFGroundDrawer* _drawer, int patchX, int patchZ)
 	smfGroundDrawer = _drawer;
 
 	// Store pointer to first byte of the height data for this patch.
-	heightMap = &(readMap->GetCornerHeightMapUnsynced())[coors.y * mapDims.mapxp1 + coors.x];
+
 
 	// Attach the two m_Base triangles together
 	baseLeft.BaseNeighbor  = &baseRight;
@@ -172,6 +185,19 @@ void Patch::Init(CSMFGroundDrawer* _drawer, int patchX, int patchZ)
 		glGenBuffersARB(1, &vertexIndexBuffer);
 	}
 
+
+	vertices.resize(3 * (PATCH_SIZE + 1) * (PATCH_SIZE + 1));
+	unsigned int index = 0;
+
+	// initialize vertices
+	for (int z = coors.y; z <= (coors.y + PATCH_SIZE); z++) {
+		for (int x = coors.x; x <= (coors.x + PATCH_SIZE); x++) {
+			vertices[index++] = x * SQUARE_SIZE;
+			vertices[index++] = 0.0f;
+			vertices[index++] = z * SQUARE_SIZE;
+		}
+	}
+
 	UpdateHeightMap();
 }
 
@@ -189,21 +215,6 @@ void Patch::Reset()
 
 void Patch::UpdateHeightMap(const SRectangle& rect)
 {
-	if (vertices.empty()) {
-		vertices.resize(3 * (PATCH_SIZE + 1) * (PATCH_SIZE + 1));
-
-		unsigned int index = 0;
-
-		// initialize vertices
-		for (int z = coors.y; z <= (coors.y + PATCH_SIZE); z++) {
-			for (int x = coors.x; x <= (coors.x + PATCH_SIZE); x++) {
-				vertices[index++] = x * SQUARE_SIZE;
-				vertices[index++] = 0.0f;
-				vertices[index++] = z * SQUARE_SIZE;
-			}
-		}
-	}
-
 	const float* hMap = readMap->GetCornerHeightMapUnsynced();
 
 	for (int z = rect.z1; z <= rect.z2; z++) {
@@ -253,9 +264,7 @@ void Patch::Split(TriTreeNode* tri)
 		Split(tri->BaseNeighbor);
 
 	// Create children and link into mesh
-	CTriNodePool* pool = CTriNodePool::GetPool();
-	tri->LeftChild  = pool->AllocateTri();
-	tri->RightChild = pool->AllocateTri();
+	currentPool->Allocate(tri->LeftChild, tri->RightChild);
 
 	// If creation failed, just exit.
 	if (!tri->IsBranch()) {
@@ -386,6 +395,12 @@ void Patch::GenerateIndices()
 	RecursRender(&baseRight, int2(PATCH_SIZE,          0), int2(         0, PATCH_SIZE), int2(PATCH_SIZE, PATCH_SIZE));
 }
 
+float Patch::GetHeight(int2 pos)
+{
+	const int vindex = (pos.y * (PATCH_SIZE + 1) + pos.x) * 3 + 1;
+	assert(readMap->GetCornerHeightMapUnsynced()[(coors.y + pos.y) * mapDims.mapxp1 + (coors.x + pos.x)] == vertices[vindex]);
+	return vertices[vindex];
+}
 
 // ---------------------------------------------------------------------
 // Computes Variance over the entire tree.  Does not examine node relationships.
@@ -409,7 +424,8 @@ float Patch::RecursComputeVariance(
 	const int2 mpos = {(left.x + rght.x) >> 1, (left.y + rght.y) >> 1};
 
 	// get the height value at M
-	const float mhgt = heightMap[(mpos.y * mapDims.mapxp1) + mpos.x];
+
+	const float mhgt = GetHeight(mpos);
 
 	// variance of this triangle is the actual height at its hypotenuse
 	// midpoint minus the interpolated height; use values passed on the
@@ -460,9 +476,9 @@ void Patch::ComputeVariance()
 		const   int2 rght = {PATCH_SIZE,          0};
 		const   int2 apex = {         0,          0};
 		const float3 hgts = {
-			heightMap[left.y * mapDims.mapxp1 + left.x],
-			heightMap[rght.y * mapDims.mapxp1 + rght.x],
-			heightMap[apex.y * mapDims.mapxp1 + apex.x],
+			GetHeight(left),
+			GetHeight(rght),
+			GetHeight(apex),
 		};
 
 		RecursComputeVariance(left, rght, apex, hgts, 1);
@@ -475,9 +491,9 @@ void Patch::ComputeVariance()
 		const   int2 rght = {         0, PATCH_SIZE};
 		const   int2 apex = {PATCH_SIZE, PATCH_SIZE};
 		const float3 hgts = {
-			heightMap[left.y * mapDims.mapxp1 + left.x],
-			heightMap[rght.y * mapDims.mapxp1 + rght.x],
-			heightMap[apex.y * mapDims.mapxp1 + apex.x],
+			GetHeight(left),
+			GetHeight(rght),
+			GetHeight(apex),
 		};
 
 		RecursComputeVariance(left, rght, apex, hgts, 1);
@@ -491,13 +507,14 @@ void Patch::ComputeVariance()
 // ---------------------------------------------------------------------
 // Create an approximate mesh.
 //
-bool Patch::Tessellate(const float3& campos, int groundDetail)
+bool Patch::Tessellate(const float3& campos, int groundDetail, bool shadowPass)
 {
 	// Set/Update LOD params (FIXME: wrong height?)
 	const float myx = (coors.x + PATCH_SIZE / 2) * SQUARE_SIZE;
 	const float myz = (coors.y + PATCH_SIZE / 2) * SQUARE_SIZE;
 	const float myy = (readMap->GetCurrMinHeight() + readMap->GetCurrMaxHeight()) * 0.5f;
 	const float3 myPos(myx, myy, myz);
+	currentPool = CTriNodePool::GetPool(shadowPass);
 
 	camDistLODFactor  = myPos.distance(campos);
 	camDistLODFactor *= 300.0f / groundDetail; // MAGIC NUMBER 1: increase the dividend to reduce LOD in camera distance
@@ -530,7 +547,7 @@ bool Patch::Tessellate(const float3& campos, int groundDetail)
 		RecursTessellate(&baseRight, left, rght, apex, 1);
 	}
 
-	return (!CTriNodePool::GetPool()->OutOfNodes());
+	return (!currentPool->OutOfNodes());
 }
 
 
