@@ -39,6 +39,16 @@ LOG_REGISTER_SECTION_GLOBAL(LOG_SECTION_SMF_GROUND_TEXTURES)
 #define LOG_SECTION_CURRENT LOG_SECTION_SMF_GROUND_TEXTURES
 
 
+CSMFGroundTextures::GroundSquare::~GroundSquare()
+{
+	glDeleteTextures(1, &textureIDs[RAW_TEX_IDX]);
+
+	textureIDs[RAW_TEX_IDX] = 0;
+	textureIDs[LUA_TEX_IDX] = 0;
+}
+
+
+
 CSMFGroundTextures::CSMFGroundTextures(CSMFReadMap* rm): smfMap(rm)
 {
 	LoadTiles(smfMap->GetFile());
@@ -46,15 +56,6 @@ CSMFGroundTextures::CSMFGroundTextures(CSMFReadMap* rm): smfMap(rm)
 	ConvolveHeightMap(mapDims.mapx, 1);
 }
 
-CSMFGroundTextures::~CSMFGroundTextures()
-{
-	for (int i = 0; i < smfMap->numBigTexX * smfMap->numBigTexY; ++i) {
-		if (!squares[i].luaTexture) {
-			glDeleteTextures(1, &squares[i].textureID);
-		}
-	}
-}
-
 void CSMFGroundTextures::LoadTiles(CSMFMapFile& file)
 {
 	loadscreen->SetLoadMessage("Loading Map Tiles");
@@ -167,12 +168,6 @@ void CSMFGroundTextures::LoadSquareTextures(const int mipLevel)
 
 	for (int y = 0; y < smfMap->numBigTexY; ++y) {
 		for (int x = 0; x < smfMap->numBigTexX; ++x) {
-			GroundSquare* square = &squares[y * smfMap->numBigTexX + x];
-			square->texLevel       = 0;
-			square->textureID      = 0;
-			square->lastBoundFrame = 1;
-			square->luaTexture     = false;
-
 			// start at the lowest mip-level
 			LoadSquareTexture(x, y, mipLevel);
 		}
@@ -277,6 +272,7 @@ bool CSMFGroundTextures::RecompressTilesIfNeeded()
 
 inline bool CSMFGroundTextures::TexSquareInView(int btx, int bty) const
 {
+	const CCamera* cam = CCamera::GetActiveCamera();
 	const float* hm = readMap->GetCornerHeightMapUnsynced();
 
 	static const float bigTexSquareRadius = fastmath::apxsqrt(
@@ -289,47 +285,48 @@ inline bool CSMFGroundTextures::TexSquareInView(int btx, int bty) const
 	const int idx = (y >> 3) * smfMap->heightMapSizeX + (x >> 3);
 	const float3 bigTexSquarePos(x, hm[idx], y);
 
-	return (cam2->InView(bigTexSquarePos, bigTexSquareRadius));
+	return (cam->InView(bigTexSquarePos, bigTexSquareRadius));
 }
 
 void CSMFGroundTextures::DrawUpdate()
 {
+	const CCamera* cam = CCamera::GetActiveCamera();
+
 	// screen-diagonal number of pixels
 	const float vsxSq = globalRendering->viewSizeX * globalRendering->viewSizeX;
 	const float vsySq = globalRendering->viewSizeY * globalRendering->viewSizeY;
 	const float vdiag = fastmath::apxsqrt(vsxSq + vsySq);
 
 	for (int y = 0; y < smfMap->numBigTexY; ++y) {
-		float dz = cam2->GetPos().z - (y * smfMap->bigSquareSize * SQUARE_SIZE);
+		float dz = cam->GetPos().z - (y * smfMap->bigSquareSize * SQUARE_SIZE);
 		dz -= (SQUARE_SIZE << 6);
 		dz = std::max(0.0f, float(math::fabs(dz) - (SQUARE_SIZE << 6)));
 
 		for (int x = 0; x < smfMap->numBigTexX; ++x) {
 			GroundSquare* square = &squares[y * smfMap->numBigTexX + x];
 
-			if (square->luaTexture) {
+			if (square->HasLuaTexture()) {
 				// no deletion or mip-level selection
 				continue;
 			}
 
 			if (!TexSquareInView(x, y)) {
-				if ((square->texLevel < 3) && (globalRendering->drawFrame - square->lastBoundFrame > 120)) {
+				if ((square->GetMipLevel() < 3) && ((globalRendering->drawFrame - square->GetDrawFrame()) > 120)) {
 					// `unload` texture (load lowest mip-map) if
 					// the square wasn't visible for 120 vframes
-					glDeleteTextures(1, &square->textureID);
 					LoadSquareTexture(x, y, 3);
 				}
 				continue;
 			}
 
-			float dx = cam2->GetPos().x - (x * smfMap->bigSquareSize * SQUARE_SIZE);
+			float dx = cam->GetPos().x - (x * smfMap->bigSquareSize * SQUARE_SIZE);
 			dx -= (SQUARE_SIZE << 6);
 			dx = std::max(0.0f, float(math::fabs(dx) - (SQUARE_SIZE << 6)));
 
 			const float hAvg =
 				(heightMaxima[y * smfMap->numBigTexX + x] +
 				 heightMinima[y * smfMap->numBigTexX + x]) / 2.0f;
-			const float dy = std::max(cam2->GetPos().y - hAvg, 0.0f);
+			const float dy = std::max(cam->GetPos().y - hAvg, 0.0f);
 			const float dist = fastmath::apxsqrt(dx * dx + dy * dy + dz * dz);
 
 			// we work under the following assumptions:
@@ -372,8 +369,7 @@ void CSMFGroundTextures::DrawUpdate()
 			if (stretchFactors[y * smfMap->numBigTexX + x] > 16000 && wantedLevel > 0)
 				wantedLevel--;
 
-			if (square->texLevel != wantedLevel) {
-				glDeleteTextures(1, &square->textureID);
+			if (square->GetMipLevel() != wantedLevel) {
 				LoadSquareTexture(x, y, wantedLevel);
 			}
 		}
@@ -389,23 +385,13 @@ bool CSMFGroundTextures::SetSquareLuaTexture(int texSquareX, int texSquareY, int
 	GroundSquare* square = &squares[texSquareY * smfMap->numBigTexX + texSquareX];
 
 	if (texID != 0) {
-		if (!square->luaTexture) {
-			// only delete textures managed by us
-			glDeleteTextures(1, &square->textureID);
-		}
-
-		square->textureID = texID;
-		square->luaTexture = true;
-	} else {
-		if (square->luaTexture) {
-			// default texture will be loaded by the next
-			// DrawUpdate (when it comes into view again)
-			square->textureID = 0;
-			square->luaTexture = false;
-		}
+		// free up some memory while the Lua texture is around
+		glDeleteTextures(1, square->GetTextureIDPtr());
+		square->SetRawTexture(0);
 	}
 
-	return (square->luaTexture);
+	square->SetLuaTexture(texID);
+	return (square->HasLuaTexture());
 }
 
 bool CSMFGroundTextures::GetSquareLuaTexture(int texSquareX, int texSquareY, int texID, int texSizeX, int texSizeY, int texMipLevel) {
@@ -483,15 +469,17 @@ void CSMFGroundTextures::LoadSquareTexture(int x, int y, int level)
 	const int numSqBytes = (mipSqSize * mipSqSize) / 2;
 
 	GroundSquare* square = &squares[y * smfMap->numBigTexX + x];
-	square->texLevel = level;
+	square->SetMipLevel(level);
+	assert(!square->HasLuaTexture());
 
 	pbo.Bind();
 	pbo.New(numSqBytes);
 	ExtractSquareTiles(x, y, level, (GLint*) pbo.MapBuffer());
 	pbo.UnmapBuffer();
 
-	glGenTextures(1, &square->textureID);
-	glBindTexture(ttarget, square->textureID);
+	glDeleteTextures(1, square->GetTextureIDPtr());
+	glGenTextures(1, square->GetTextureIDPtr());
+	glBindTexture(ttarget, square->GetTextureID());
 	glTexParameteri(ttarget, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glTexParameteri(ttarget, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 
@@ -500,8 +488,8 @@ void CSMFGroundTextures::LoadSquareTexture(int x, int y, int level)
 		glTexParameteri(ttarget, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 	}
 
-	if (smfMap->GetAnisotropy() != 0.0f)
-		glTexParameterf(ttarget, GL_TEXTURE_MAX_ANISOTROPY_EXT, smfMap->GetAnisotropy());
+	if (smfMap->GetTexAnisotropyLevel(false) != 0.0f)
+		glTexParameterf(ttarget, GL_TEXTURE_MAX_ANISOTROPY_EXT, smfMap->GetTexAnisotropyLevel(false));
 
 	if (level < 2) {
 		glTexParameteri(ttarget, GL_TEXTURE_PRIORITY, 1);
@@ -523,9 +511,9 @@ void CSMFGroundTextures::BindSquareTexture(int texSquareX, int texSquareY)
 	assert(texSquareY < smfMap->numBigTexY);
 
 	GroundSquare* square = &squares[texSquareY * smfMap->numBigTexX + texSquareX];
-	glBindTexture(GL_TEXTURE_2D, square->textureID);
+	glBindTexture(GL_TEXTURE_2D, square->GetTextureID());
 
 	if (game->GetDrawMode() == CGame::gameNormalDraw) {
-		square->lastBoundFrame = globalRendering->drawFrame;
+		square->SetDrawFrame(globalRendering->drawFrame);
 	}
 }
