@@ -3,6 +3,8 @@
 #ifndef _SMF_GROUND_TEXTURES_H_
 #define _SMF_GROUND_TEXTURES_H_
 
+#include <vector>
+
 #include "Map/BaseGroundTextures.h"
 #include "Rendering/GL/PBO.h"
 
@@ -13,7 +15,6 @@ class CSMFGroundTextures: public CBaseGroundTextures
 {
 public:
 	CSMFGroundTextures(CSMFReadMap* rm);
-	~CSMFGroundTextures();
 
 	void DrawUpdate();
 	bool SetSquareLuaTexture(int texSquareX, int texSquareY, int texID);
@@ -34,10 +35,30 @@ protected:
 
 private:
 	struct GroundSquare {
-		unsigned int texLevel;
-		unsigned int textureID;
-		unsigned int lastBoundFrame;
-		bool luaTexture;
+		enum {
+			RAW_TEX_IDX = 0,
+			LUA_TEX_IDX = 1,
+		};
+
+		GroundSquare(): textureIDs{0, 0}, texMipLevel(0), texDrawFrame(1) {}
+		~GroundSquare();
+
+		bool HasLuaTexture() const { return (textureIDs[LUA_TEX_IDX] != 0); }
+
+		void SetRawTexture(unsigned int id) { textureIDs[RAW_TEX_IDX] = id; }
+		void SetLuaTexture(unsigned int id) { textureIDs[LUA_TEX_IDX] = id; }
+		void SetMipLevel(unsigned int l) { texMipLevel = l; }
+		void SetDrawFrame(unsigned int f) { texDrawFrame = f; }
+
+		unsigned int* GetTextureIDPtr() { return &textureIDs[RAW_TEX_IDX]; }
+		unsigned int GetTextureID() const { return textureIDs[HasLuaTexture()]; }
+		unsigned int GetMipLevel() const { return texMipLevel; }
+		unsigned int GetDrawFrame() const { return texDrawFrame; }
+
+	private:
+		unsigned int textureIDs[2];
+		unsigned int texMipLevel;
+		unsigned int texDrawFrame;
 	};
 
 	std::vector<GroundSquare> squares;
