@@ -4,6 +4,7 @@
 #define SELECTIONWIDGET_H
 
 #include <string>
+#include <vector>
 #include <boost/bind.hpp>
 
 #include "aGui/GuiElement.h"
@@ -83,7 +84,8 @@ public:
 
 private:
 	void CleanWindow();
-
+	void UpdateAvailableScripts();
+	
 	agui::Button* mod;
 	agui::TextElement* modT;
 	agui::Button* map;
@@ -91,6 +93,7 @@ private:
 	agui::Button* script;
 	agui::TextElement* scriptT;
 	ListSelectWnd* curSelect;
+	std::vector<std::string> availableScripts;
 };
 
 #endif
