@@ -64,8 +64,8 @@ CBasicSky::CBasicSky()
 	rawClouds=newmat2<int>(CLOUD_SIZE,CLOUD_SIZE);
 	blendMatrix=newmat3<int>(CLOUD_DETAIL,32,32);
 
-	domeheight=math::cos(PI/16)*1.01f;
-	domeWidth=math::sin(PI/16)*400*1.7f;
+	domeheight=std::cos(PI/16)*1.01f;
+	domeWidth=std::sin(PI/16)*400*1.7f;
 
 	UpdateSkyDir();
 	InitSun();
@@ -220,6 +220,8 @@ CBasicSky::~CBasicSky()
 
 void CBasicSky::Draw()
 {
+	SCOPED_GMARKER("CBasicSky::Draw");
+
 	if (!globalRendering->drawSky)
 		return;
 
@@ -235,8 +237,8 @@ void CBasicSky::Draw()
 		glTranslatef((gs->frameNum%20000)*0.00005f+modCamera.x*0.000025f,modCamera.z*0.000025f,0);
 	glMatrixMode(GL_MODELVIEW);
 		glPushMatrix();
-		CMatrix44f m(camera->GetPos(),skydir1,UpVector,skydir2);
-		glMultMatrixf(m.m);
+		// glTranslatef3(camera->GetPos());
+		glMultMatrixf(CMatrix44f(camera->GetPos(), skydir1, UpVector, skydir2));
 
 	glCallList(skyDomeList);
 
@@ -369,7 +371,7 @@ void CBasicSky::Update()
 	case 0: {
 		for(int a=0; a<CLOUD_DETAIL; a++) {
 			float fade = gs->frameNum / (70.0f * (2<<(CLOUD_DETAIL-1-a)));
-			fade -= math::floor(fade/2)*2;
+			fade -= std::floor(fade/2)*2;
 			if(fade>1) {
 				fade = 2 - fade;
 				if(!cloudDown[a]) {
@@ -403,11 +405,11 @@ void CBasicSky::Update()
 		int qcda=(4<<CLOUD_DETAIL)>>a;
 		int *pkernel=kernel;
 		for(int y=0; y<cs4a; ++y, pkernel+=CLOUD_SIZE/4) {
-			float ydist=math::fabs(1.0f+y-cs8a)/cs8a;
+			float ydist=std::fabs(1.0f+y-cs8a)/cs8a;
 			ydist=ydist*ydist*(3-2*ydist);
 			int *pkrn=pkernel;
 			for(int x=0; x<cs4a; ++x) {
-				float xdist=math::fabs(1.0f+x-cs8a)/cs8a;
+				float xdist=std::fabs(1.0f+x-cs8a)/cs8a;
 				xdist=xdist*xdist*(3-2*xdist);
 
 				float contrib=(1-xdist)*(1-ydist);
@@ -506,7 +508,7 @@ void CBasicSky::CreateTransformVectors()
 	unsigned char *tt=thicknessTransform;
 	for(int a=0;a<1024;++a){
 		float f=(1023.0f-(a+cloudDensity*1024-512))/1023.0f;
-		float alpha=math::pow(f*2,3);
+		float alpha=std::pow(f*2,3);
 		if(alpha>1)
 			alpha=1;
 		*at=(unsigned char)(alpha*255);
@@ -518,6 +520,8 @@ void CBasicSky::CreateTransformVectors()
 
 void CBasicSky::DrawSun()
 {
+	SCOPED_GMARKER("CBasicSky::DrawSun");
+
 	if (!globalRendering->drawSky)
 		return;
 	if (!SunVisible(camera->GetPos()))
@@ -543,8 +547,8 @@ void CBasicSky::DrawSun()
 	const float ymod = (sunTexCoordY - 0.5f) * domeWidth * 0.025f * 256;
 	const float fx = gs->frameNum * 0.00005f * CLOUD_SIZE + xzSunCameraPos.x * CLOUD_SIZE * 0.000025f;
 	const float fy = ymod + xzSunCameraPos.z * CLOUD_SIZE * 0.000025f;
-	const float ffx = fx - math::floor(fx);
-	const float ffy = fy - math::floor(fy);
+	const float ffx = fx - std::floor(fx);
+	const float ffy = fy - std::floor(fy);
 	const int baseX = int(fx) & CLOUD_MASK;
 	const int baseY = int(fy) & CLOUD_MASK;
 
@@ -604,8 +608,8 @@ void CBasicSky::UpdateSunFlare() {
 		glBegin(GL_TRIANGLE_STRIP);
 
 		for (int x = 0; x < 257; ++x) {
-			const float dx = math::sin(x * 2.0f * PI / 256.0f);
-			const float dy = math::cos(x * 2.0f * PI / 256.0f);
+			const float dx = std::sin(x * 2.0f * PI / 256.0f);
+			const float dy = std::cos(x * 2.0f * PI / 256.0f);
 			const float dz = 5.0f;
 
 			glTexCoord2f(x / 256.0f, 0.25f); glVertexf3(zdir * dz + xdir * dx * 0.0014f + ydir * dy * 0.0014f);
@@ -706,7 +710,7 @@ void CBasicSky::UpdateSunDir() {
 	// polar-coordinate direction in yz-plane (sin(zenith-angle), radius)
 	modSunDir.x = 0.0f;
 	modSunDir.y = L.y;
-	modSunDir.z = math::sqrt(L.x * L.x + L.z * L.z);
+	modSunDir.z = std::sqrt(L.x * L.x + L.z * L.z);
 
 	sunTexCoordX = 0.5f;
 	sunTexCoordY = GetTexCoordFromDir(modSunDir);
@@ -757,12 +761,12 @@ float3 CBasicSky::GetDirFromTexCoord(float x, float y)
 	dir.x = (x - 0.5f) * domeWidth;
 	dir.z = (y - 0.5f) * domeWidth;
 
-	const float hdist = math::sqrt(dir.x * dir.x + dir.z * dir.z);
+	const float hdist = std::sqrt(dir.x * dir.x + dir.z * dir.z);
 	const float ang = GetRadFromXY(dir.x, dir.z) + skyAngle;
-	const float fy = math::asin(hdist / 400);
+	const float fy = std::asin(hdist / 400);
 
-	dir.x = hdist * math::cos(ang);
-	dir.z = hdist * math::sin(ang);
+	dir.x = hdist * std::cos(ang);
+	dir.z = hdist * std::sin(ang);
 	dir.y = (fastmath::cos(fy) - domeheight) * 400;
 
 	dir.ANormalize();
@@ -795,8 +799,8 @@ void CBasicSky::UpdateTexPartDot3(int x, int y, unsigned char (*texp)[4]) {
 	const float3& dir = GetDirFromTexCoord(x / 256.0f, (255.0f - y) / 256.0f);
 
 	const float sunInt = skyLight->GetLightIntensity();
-	const float sunDist = math::acos(dir.dot(skyLight->GetLightDir())) * 50;
-	const float sunMod = sunInt * (0.3f / math::sqrt(sunDist) + 2.0f / sunDist);
+	const float sunDist = std::acos(dir.dot(skyLight->GetLightDir())) * 50;
+	const float sunMod = sunInt * (0.3f / std::sqrt(sunDist) + 2.0f / sunDist);
 
 	const float green = std::min(1.0f, (0.55f + sunMod));
 
@@ -809,7 +813,7 @@ void CBasicSky::UpdateTexPartDot3(int x, int y, unsigned char (*texp)[4]) {
 void CBasicSky::UpdateTexPart(int x, int y, unsigned char (*texp)[4]) {
 	const float3& dir = GetDirFromTexCoord(x / 512.0f, (511.0f - y) / 512.0f);
 
-	const float sunDist = math::acos(dir.dot(skyLight->GetLightDir())) * 70;
+	const float sunDist = std::acos(dir.dot(skyLight->GetLightDir())) * 70;
 	const float sunMod = skyLight->GetLightIntensity() * 12.0f / (12 + sunDist);
 
 	const float red   = std::min(skyColor.x + sunMod * sunColor.x, 1.0f);
