@@ -3,7 +3,6 @@
 #ifndef _BASIC_TREE_DRAWER_H_
 #define _BASIC_TREE_DRAWER_H_
 
-#include <map>
 #include "ITreeDrawer.h"
 #include "Rendering/GL/myGL.h"
 
@@ -18,16 +17,7 @@ public:
 	virtual ~CBasicTreeDrawer();
 
 	void Draw(float treeDistance, bool drawReflection);
-	void Update();
-	void ResetPos(const float3& pos);
-	void AddTree(int treeID, int treeType, const float3& pos, float size);
-	void DeleteTree(int treeID, const float3& pos);
-
-	TreeSquareStruct* trees;
-
-	int treesX;
-	int treesY;
-	int nTrees;
+	void Update() {}
 
 private:
 	GLuint treetex;
