@@ -296,7 +296,7 @@ void CubeMapHandler::CreateSpecularFacePart(
 	for (int x = 0; x < size; ++x) {
 		const float3 dir = (cdir + (xdif * (x + 0.5f)) / size + (ydif * (y + 0.5f)) / size).Normalize();
 		const float dot  = std::max(0.0f, dir.dot(sky->GetLight()->GetLightDir()));
-		const float spec = std::min(1.0f, math::pow(dot, sunLighting->specularExponent) + math::pow(dot, 3.0f) * 0.25f);
+		const float spec = std::min(1.0f, std::pow(dot, sunLighting->specularExponent) + std::pow(dot, 3.0f) * 0.25f);
 
 		buf[x * 4 + 0] = (sunLighting->unitSpecularColor.x * spec * 255);
 		buf[x * 4 + 1] = (sunLighting->unitSpecularColor.y * spec * 255);
