@@ -7,18 +7,22 @@
 
 #define CLOUD_SIZE 256 // must be divisible by 4 and 8
 
+struct MapTextureData;
 class ISky
 {
 public:
 	static ISky* GetSky();
 
 	virtual ~ISky();
+
 	virtual void Update() = 0;
+	virtual void UpdateSunDir() = 0;
+	virtual void UpdateSkyTexture() = 0;
+
 	virtual void Draw() = 0;
 	virtual void DrawSun() = 0;
 
-	virtual void UpdateSunDir() = 0;
-	virtual void UpdateSkyTexture() = 0;
+	virtual void SetLuaTexture(const MapTextureData& td) {}
 
 	void IncreaseCloudDensity() { cloudDensity *= 1.05f; }
 	void DecreaseCloudDensity() { cloudDensity *= 0.95f; }
@@ -33,7 +37,7 @@ public:
 	 * Sets up OpenGL to draw fog or not, according to the value of
 	 * globalRendering->drawFog.
 	 */
-	static void SetupFog();
+	void SetupFog();
 
 public:
 	bool wireframe;
@@ -41,17 +45,20 @@ public:
 
 	float3 sundir1, sundir2; // (xvec, yvec) TODO: move these to SkyLight
 	float3 modSunDir;
+
 	float3 skyColor;
 	float3 sunColor;
 	float3 cloudColor;
+	float4 fogColor;
+
 	float fogStart;
+	float fogEnd;
+	float cloudDensity;
 
 protected:
 	ISky();
 
 	ISkyLight* skyLight;
-
-	float cloudDensity;
 };
 
 extern ISky* sky;
