@@ -48,7 +48,7 @@ void DynamicSkyLight::Update() {
 	if (!luaControl) {
 		SetLightDir(CalculateSunPos(sunStartAngle).ANormalize());
 
-		lightIntensity = math::sqrt(Clamp(lightDir.y, 0.0f, 1.0f));
+		lightIntensity = std::sqrt(Clamp(lightDir.y, 0.0f, 1.0f));
 
 		const float shadowDensity = std::min(1.0f, lightIntensity * shadowDensityFactor);
 		const CMapInfo::light_t& light = mapInfo->light;
@@ -61,7 +61,7 @@ void DynamicSkyLight::Update() {
 		updateNeeded = false;
 
 		sky->UpdateSunDir();
-		eventHandler.SunChanged(lightDir);
+		eventHandler.SunChanged();
 	}
 }
 
@@ -70,7 +70,7 @@ void DynamicSkyLight::Update() {
 bool DynamicSkyLight::SetLightDir(const float4& newLightDir) {
 	if (newLightDir != lightDir) {
 		static float4 lastUpdate = ZeroVector;
-		static const float minCosAngle = math::cos(1.5f * (PI/180.f));
+		static const float minCosAngle = std::cos(1.5f * (PI/180.f));
 
 		if (lastUpdate.dot(newLightDir) < minCosAngle) {
 			lastUpdate   = newLightDir;
@@ -90,7 +90,7 @@ float4 DynamicSkyLight::CalculateSunPos(const float startAngle) const {
 	const float angularVelocity = 2.0f * PI / sunOrbitTime;
 
 	const float sunAng  = startAngle - initialSunAngle - angularVelocity * gameSeconds;
-	const float4 sunPos = sunRotation.Mul(float3(sunOrbitRad * math::cos(sunAng), sunOrbitHeight, sunOrbitRad * math::sin(sunAng)));
+	const float4 sunPos = sunRotation.Mul(float3(sunOrbitRad * std::cos(sunAng), sunOrbitHeight, sunOrbitRad * std::sin(sunAng)));
 
 	return sunPos;
 }
@@ -106,18 +106,18 @@ void DynamicSkyLight::SetLightParams(float4 newLightDir, float startAngle, float
 	if (newLightDir.w == FLT_MAX) {
 		// old: newLightDir is position where sun reaches highest altitude
 		const float sunLen = newLightDir.Length2D();
-		const float sunAzimuth = (sunLen <= 0.001f) ? PI / 2.0f : math::atan(newLightDir.y / sunLen);
-		const float sunHeight = math::tan(sunAzimuth - 0.001f);
+		const float sunAzimuth = (sunLen <= 0.001f) ? PI / 2.0f : std::atan(newLightDir.y / sunLen);
+		const float sunHeight = std::tan(sunAzimuth - 0.001f);
 
-		float3 v1(math::cos(initialSunAngle), sunHeight, math::sin(initialSunAngle));
+		float3 v1(std::cos(initialSunAngle), sunHeight, std::sin(initialSunAngle));
 		v1.ANormalize();
 
 		if (v1.y <= orbitMinSunHeight) {
 			newLightDir = UpVector;
 			sunOrbitHeight = v1.y;
-			sunOrbitRad = math::sqrt(1.0f - sunOrbitHeight * sunOrbitHeight);
+			sunOrbitRad = std::sqrt(1.0f - sunOrbitHeight * sunOrbitHeight);
 		} else {
-			float3 v2(math::cos(initialSunAngle + PI), orbitMinSunHeight, math::sin(initialSunAngle + PI));
+			float3 v2(std::cos(initialSunAngle + PI), orbitMinSunHeight, std::sin(initialSunAngle + PI));
 			v2.ANormalize();
 			float3 v3 = v2 - v1;
 			sunOrbitRad = v3.Length() / 2.0f;
@@ -134,8 +134,8 @@ void DynamicSkyLight::SetLightParams(float4 newLightDir, float startAngle, float
 		}
 	} else {
 		// new: newLightDir is center position of orbit, and newLightDir.w is orbit height
-		sunOrbitHeight = std::max(-1.0f, std::min(newLightDir.w, 1.0f));
-		sunOrbitRad = math::sqrt(1.0f - sunOrbitHeight * sunOrbitHeight);
+		sunOrbitHeight = Clamp(newLightDir.w, -1.0f, 1.0f);
+		sunOrbitRad = std::sqrt(1.0f - sunOrbitHeight * sunOrbitHeight);
 	}
 
 	sunRotation.LoadIdentity();
