@@ -14,24 +14,24 @@
 IPathDrawer* pathDrawer = NULL;
 
 IPathDrawer* IPathDrawer::GetInstance() {
-	static IPathDrawer* pd = NULL;
-
-	if (pd == NULL) {
+	if (pathDrawer == NULL) {
 		if (dynamic_cast<QTPFS::PathManager*>(pathManager) != NULL) {
-			return (pd = new QTPFSPathDrawer());
+			return (pathDrawer = new QTPFSPathDrawer());
 		}
 		if (dynamic_cast<CPathManager*>(pathManager) != NULL) {
-			return (pd = new DefaultPathDrawer());
+			return (pathDrawer = new DefaultPathDrawer());
 		}
 
-		pd = new IPathDrawer();
+		pathDrawer = new IPathDrawer();
 	}
 
-	return pd;
+	return pathDrawer;
 }
 
 void IPathDrawer::FreeInstance(IPathDrawer* pd) {
+	assert(pd == pathDrawer);
 	delete pd;
+	pathDrawer = NULL;
 }
 
 
@@ -71,8 +71,8 @@ SColor IPathDrawer::GetSpeedModColor(const float sm) {
 float IPathDrawer::GetSpeedModNoObstacles(const MoveDef* md, int sqx, int sqz) {
 	float m = 0.0f;
 
-	const int hmIdx = sqz * gs->mapxp1 + sqx;
-	const int cnIdx = sqz * gs->mapx   + sqx;
+	const int hmIdx = sqz * mapDims.mapxp1 + sqx;
+	const int cnIdx = sqz * mapDims.mapx   + sqx;
 
 	const float height = hm[hmIdx];
 	const float slope = 1.0f - cn[cnIdx].y;
