@@ -377,10 +377,10 @@ void CShadowHandler::SetShadowMapSizeFactors()
 {
 	#if (SHADOWMATRIX_NONLINEAR == 1)
 	// note: depends on CalcMinMaxView(), which is no longer called
-	const float shadowMapX =              math::sqrt( math::fabs(shadowProjMinMax.y) ); // math::sqrt( |x2| )
-	const float shadowMapY =              math::sqrt( math::fabs(shadowProjMinMax.w) ); // math::sqrt( |y2| )
-	const float shadowMapW = shadowMapX + math::sqrt( math::fabs(shadowProjMinMax.x) ); // math::sqrt( |x2| ) + math::sqrt( |x1| )
-	const float shadowMapH = shadowMapY + math::sqrt( math::fabs(shadowProjMinMax.z) ); // math::sqrt( |y2| ) + math::sqrt( |y1| )
+	const float shadowMapX =              std::sqrt( std::fabs(shadowProjMinMax.y) ); // math::sqrt( |x2| )
+	const float shadowMapY =              std::sqrt( std::fabs(shadowProjMinMax.w) ); // math::sqrt( |y2| )
+	const float shadowMapW = shadowMapX + std::sqrt( std::fabs(shadowProjMinMax.x) ); // math::sqrt( |x2| ) + math::sqrt( |x1| )
+	const float shadowMapH = shadowMapY + std::sqrt( std::fabs(shadowProjMinMax.z) ); // math::sqrt( |y2| ) + math::sqrt( |y1| )
 
 	shadowTexProjCenter.x = 1.0f - (shadowMapX / shadowMapW);
 	shadowTexProjCenter.y = 1.0f - (shadowMapY / shadowMapH);
@@ -488,6 +488,46 @@ void CShadowHandler::SetShadowCamera(CCamera* shadowCam)
 	shadowCam->UpdateFrustum();
 }
 
+
+void CShadowHandler::SetupShadowTexSampler(unsigned int texUnit, bool enable) const
+{
+	glActiveTexture(texUnit);
+	glBindTexture(GL_TEXTURE_2D, shadowTexture);
+
+	// support FFP context
+	if (enable)
+		glEnable(GL_TEXTURE_2D);
+
+	SetupShadowTexSamplerRaw();
+}
+
+void CShadowHandler::SetupShadowTexSamplerRaw() const
+{
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_COMPARE_R_TO_TEXTURE);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC, GL_LEQUAL);
+	glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE, GL_LUMINANCE);
+	// glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE, GL_INTENSITY);
+	// glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE, GL_ALPHA);
+}
+
+void CShadowHandler::ResetShadowTexSampler(unsigned int texUnit, bool disable) const
+{
+	glActiveTexture(texUnit);
+	glBindTexture(GL_TEXTURE_2D, 0);
+
+	if (disable)
+		glDisable(GL_TEXTURE_2D);
+
+	ResetShadowTexSamplerRaw();
+}
+
+void CShadowHandler::ResetShadowTexSamplerRaw() const
+{
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_NONE);
+	glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE, GL_LUMINANCE);
+}
+
+
 void CShadowHandler::CreateShadows()
 {
 	// NOTE:
@@ -703,7 +743,7 @@ float CShadowHandler::GetOrthoProjectedFrustumRadius(CCamera* cam, float3& projP
 	}
 
 	const float maxMapDiameter = readMap->GetBoundingRadius() * 2.0f;
-	const float frustumDiameter = math::sqrt(frustumCenter.w) * 2.0f;
+	const float frustumDiameter = std::sqrt(frustumCenter.w) * 2.0f;
 
 	return (std::min(maxMapDiameter, frustumDiameter));
 }
