@@ -6,7 +6,6 @@
 #include <memory>
 #include <string.h>
 #include <IL/il.h>
-//#include <IL/ilu.h>
 #include <SDL_video.h>
 #include <boost/thread.hpp>
 
@@ -225,28 +224,27 @@ bool CBitmap::Load(std::string const& filename, unsigned char defaultAlpha)
 		channels = 0;
 
 		ddsimage = new nv_dds::CDDSImage();
-		bool status = ddsimage->load(filename);
+		if (!ddsimage->load(filename))
+			return false;
 
-		if (status) {
-			xsize = ddsimage->get_width();
-			ysize = ddsimage->get_height();
-			channels = ddsimage->get_components();
-			switch (ddsimage->get_type()) {
-				case nv_dds::TextureFlat :
-					textype = GL_TEXTURE_2D;
-					break;
-				case nv_dds::Texture3D :
-					textype = GL_TEXTURE_3D;
-					break;
-				case nv_dds::TextureCubemap :
-					textype = GL_TEXTURE_CUBE_MAP;
-					break;
-				case nv_dds::TextureNone :
-				default :
-					break;
-			}
+		xsize = ddsimage->get_width();
+		ysize = ddsimage->get_height();
+		channels = ddsimage->get_components();
+		switch (ddsimage->get_type()) {
+			case nv_dds::TextureFlat :
+				textype = GL_TEXTURE_2D;
+				break;
+			case nv_dds::Texture3D :
+				textype = GL_TEXTURE_3D;
+				break;
+			case nv_dds::TextureCubemap :
+				textype = GL_TEXTURE_CUBE_MAP;
+				break;
+			case nv_dds::TextureNone :
+			default :
+				break;
 		}
-		return status;
+		return true;
 #else // !BITMAP_NO_OPENGL
 		AllocDummy(); //allocate a dummy texture, as dds aren't supported in headless
 		return true;
@@ -463,7 +461,7 @@ bool CBitmap::SaveFloat(std::string const& filename) const
 unsigned int CBitmap::CreateTexture(float aniso, bool mipmaps) const
 {
 	if (compressed)
-		return CreateDDSTexture(0, mipmaps, aniso);
+		return CreateDDSTexture(0, aniso, mipmaps);
 
 	if (mem.empty())
 		return 0;
@@ -521,7 +519,7 @@ static void HandleDDSMipmap(GLenum target, bool mipmaps, int num_mipmaps)
 	}
 }
 
-unsigned int CBitmap::CreateDDSTexture(unsigned int texID, bool mipmaps, float aniso) const
+unsigned int CBitmap::CreateDDSTexture(unsigned int texID, float aniso, bool mipmaps) const
 {
 	glPushAttrib(GL_TEXTURE_BIT);
 
@@ -583,7 +581,7 @@ unsigned int CBitmap::CreateTexture(float aniso, bool mipmaps) const {
 	return 0;
 }
 
-unsigned int CBitmap::CreateDDSTexture(unsigned int texID, bool mipmaps, float aniso) const {
+unsigned int CBitmap::CreateDDSTexture(unsigned int texID, float aniso, bool mipmaps) const {
 	return 0;
 }
 #endif // !BITMAP_NO_OPENGL
