@@ -5,10 +5,11 @@
 
 #include <string>
 #include <vector>
-#include <list>
 #include <map>
+#include <deque>
 
 #include "System/creg/creg_cond.h"
+#include "System/Color.h"
 
 /**
  * Simple class to interpolate between 32bit RGBA colors
@@ -24,10 +25,15 @@ public:
 	CColorMap(const std::vector<float>& vec);
 	/// Loads from a file
 	CColorMap(const std::string& fileName);
-	/// Loads from a char buffer as one dimensional array
-	CColorMap(const unsigned char* buf, int num);
-	~CColorMap();
 
+public:
+	/**
+	 * @param color buffer with room for 4 bytes
+	 * @param pos value between 0.0f and 1.0f, returns pointer to color
+	 */
+	void GetColor(unsigned char* color, float pos);
+
+public:
 	/// Load colormap from a bitmap
 	static CColorMap* LoadFromBitmapFile(const std::string& fileName);
 
@@ -35,26 +41,13 @@ public:
 	static CColorMap* LoadFromFloatVector(const std::vector<float>& vec);
 
 	/**
-	 * Load from a string containing a number of float values.
+	 * Load from a string containing a number of float values or filename.
 	 * example: "1.0 0.5 1.0 ... "
 	 */
-	static CColorMap* LoadFromFloatString(const std::string& fString);
-
 	static CColorMap* LoadFromDefString(const std::string& dString);
 
-	/// To load default values
-	static CColorMap* Load12f(float r1, float g1, float b1, float a1, float r2, float g2, float b2, float a2, float r3, float g3, float b3, float a3);
-
-	/**
-	 * @param color buffer with room for 4 bytes
-	 * @param pos value between 0.0f and 1.0f, returns pointer to color
-	 */
-	unsigned char* GetColor(unsigned char* color, float pos);
-
-	static void DeleteColormaps();
-
 protected:
-	std::vector<unsigned char> map;
+	std::vector<SColor> map;
 	int xsize;
 	int nxsize;
 	int ysize;
@@ -62,7 +55,7 @@ protected:
 
 	void LoadMap(const unsigned char* buf, int num);
 
-	static std::vector<CColorMap*> colorMaps;
+	static std::deque<CColorMap> colorMaps;
 	static std::map<std::string, CColorMap*> colorMapsMap;
 };
 
