@@ -11,18 +11,20 @@
 CR_BIND(CollisionVolume, )
 CR_REG_METADATA(CollisionVolume, (
 	CR_MEMBER(fullAxisScales),
-	CR_MEMBER(halfAxisScales),
-	CR_MEMBER(halfAxisScalesSqr),
-	CR_MEMBER(halfAxisScalesInv),
+	CR_IGNORED(halfAxisScales),
+	CR_IGNORED(halfAxisScalesSqr),
+	CR_IGNORED(halfAxisScalesInv),
 	CR_MEMBER(axisOffsets),
-	CR_MEMBER(volumeBoundingRadius),
-	CR_MEMBER(volumeBoundingRadiusSq),
+	CR_IGNORED(volumeBoundingRadius),
+	CR_IGNORED(volumeBoundingRadiusSq),
 	CR_MEMBER(volumeType),
 	CR_MEMBER(volumeAxes),
 	CR_MEMBER(ignoreHits),
 	CR_MEMBER(useContHitTest),
 	CR_MEMBER(defaultToFootPrint),
-	CR_MEMBER(defaultToPieceTree)
+	CR_MEMBER(defaultToPieceTree),
+
+	CR_POSTLOAD(PostLoad)
 ))
 
 // base ctor (CREG-only)
@@ -105,6 +107,12 @@ CollisionVolume::CollisionVolume(
 }
 
 
+void CollisionVolume::PostLoad()
+{
+	SetAxisScales(fullAxisScales);
+	SetBoundingRadius();
+}
+
 
 void CollisionVolume::InitSphere(float radius)
 {
@@ -256,7 +264,7 @@ float CollisionVolume::GetPointSurfaceDistance(const CUnit* u, const LocalModelP
 }
 
 float CollisionVolume::GetPointSurfaceDistance(const CFeature* f, const LocalModelPiece* lmp, const float3& pos) const {
-	return (GetPointSurfaceDistance(f, lmp, f->GetTransformMatrixRef(), pos));
+	return (GetPointSurfaceDistance(f, lmp, f->GetTransformMatrixRef(true), pos));
 }
 
 float CollisionVolume::GetPointSurfaceDistance(
