@@ -19,34 +19,25 @@
  * Global instance of CGlobalSynced
  */
 CGlobalSynced* gs = NULL;
-
+CGlobalSynced::SyncedRNG CGlobalSynced::rng;
 
 
 CR_BIND(CGlobalSynced, )
 
 CR_REG_METADATA(CGlobalSynced, (
 	CR_MEMBER(frameNum),
+	CR_MEMBER(tempNum),
+
 	CR_MEMBER(speedFactor),
 	CR_MEMBER(wantedSpeedFactor),
+
 	CR_MEMBER(paused),
-	CR_MEMBER(mapx),
-	CR_MEMBER(mapxm1),
-	CR_MEMBER(mapxp1),
-	CR_MEMBER(mapy),
-	CR_MEMBER(mapym1),
-	CR_MEMBER(mapyp1),
-	CR_MEMBER(mapSquares),
-	CR_MEMBER(hmapx),
-	CR_MEMBER(hmapy),
-	CR_MEMBER(pwr2mapx),
-	CR_MEMBER(pwr2mapy),
-	CR_MEMBER(tempNum),
 	CR_MEMBER(godMode),
-	CR_MEMBER(globalLOS),
 	CR_MEMBER(cheatEnabled),
 	CR_MEMBER(noHelperAIs),
 	CR_MEMBER(editDefsEnabled),
 	CR_MEMBER(useLuaGaia),
+
 	CR_MEMBER(randSeed),
 	CR_MEMBER(initRandSeed)
 ))
@@ -57,52 +48,57 @@ CR_REG_METADATA(CGlobalSynced, (
  */
 CGlobalSynced::CGlobalSynced()
 {
-	mapx  = 512;
-	mapy  = 512;
-	mapxm1 = mapx - 1;
-	mapym1 = mapy - 1;
-	mapxp1 = mapx + 1;
-	mapyp1 = mapy + 1;
-	mapSquares = mapx * mapy;
-	hmapx = mapx>>1;
-	hmapy = mapy>>1;
-	pwr2mapx = mapx; //next_power_of_2(mapx);
-	pwr2mapy = mapy; //next_power_of_2(mapy);
-	randSeed = 18655;
+	randSeed     = 18655;
 	initRandSeed = randSeed;
-	frameNum = 0;
-	speedFactor = 1;
-	wantedSpeedFactor = 1;
-	paused = false;
-	godMode = false;
-	cheatEnabled = false;
-	noHelperAIs = false;
-	editDefsEnabled = false;
-	tempNum = 2;
-	useLuaGaia = true;
 
-	memset(globalLOS, 0, sizeof(globalLOS));
-	log_framePrefixer_setFrameNumReference(&frameNum);
-
-	teamHandler = new CTeamHandler();
+	assert(teamHandler == NULL);
+	ResetState();
 }
 
 
 CGlobalSynced::~CGlobalSynced()
 {
 	SafeDelete(teamHandler);
+	assert(teamHandler == NULL);
 
 	log_framePrefixer_setFrameNumReference(NULL);
 }
 
 
+void CGlobalSynced::ResetState() {
+	frameNum = -1; // first real frame is 0
+	tempNum  =  1;
+
+	speedFactor       = 1.0f;
+	wantedSpeedFactor = 1.0f;
+
+	paused  = false;
+	godMode = false;
+
+	cheatEnabled    = false;
+	noHelperAIs     = false;
+	editDefsEnabled = false;
+	useLuaGaia      = true;
+
+	log_framePrefixer_setFrameNumReference(&frameNum);
+
+	if (teamHandler == NULL) {
+		// needs to be available as early as PreGame
+		teamHandler = new CTeamHandler();
+	} else {
+		// less cavemanly than delete + new
+		teamHandler->ResetState();
+		skirmishAIHandler.ResetState();
+	}
+}
+
 void CGlobalSynced::LoadFromSetup(const CGameSetup* setup)
 {
-	noHelperAIs = setup->noHelperAIs;
-	useLuaGaia  = setup->useLuaGaia;
+	noHelperAIs     = setup->noHelperAIs;
+	useLuaGaia      = setup->useLuaGaia;
 
-	skirmishAIHandler.LoadFromSetup(*setup);
 	teamHandler->LoadFromSetup(setup);
+	skirmishAIHandler.LoadFromSetup(*setup);
 }
 
 /**
@@ -135,11 +131,12 @@ float CGlobalSynced::randFloat()
 float3 CGlobalSynced::randVector()
 {
 	float3 ret;
+
 	do {
-		ret.x = randFloat()*2-1;
-		ret.y = randFloat()*2-1;
-		ret.z = randFloat()*2-1;
-	} while(ret.SqLength()>1);
+		ret.x = randFloat() * 2.0f - 1.0f;
+		ret.y = randFloat() * 2.0f - 1.0f;
+		ret.z = randFloat() * 2.0f - 1.0f;
+	} while (ret.SqLength() > 1.0f);
 
 	return ret;
 }
