@@ -35,9 +35,9 @@ needed to support dynamic resizing (not used yet)
 	CQuadField(int2 mapDims, int quad_size);
 	~CQuadField();
 
-	std::vector<int> GetQuads(float3 pos, const float radius);
-	std::vector<int> GetQuadsRectangle(const float3 mins, const float3 maxs);
-	std::vector<int> GetQuadsOnRay(const float3 start, const float3 dir, const float length);
+	const std::vector<int>& GetQuads(float3 pos, float radius);
+	const std::vector<int>& GetQuadsRectangle(const float3& mins, const float3& maxs);
+	const std::vector<int>& GetQuadsOnRay(const float3& start, const float3& dir, float length);
 
 	void GetUnitsAndFeaturesColVol(
 		const float3& pos,
@@ -51,34 +51,41 @@ needed to support dynamic resizing (not used yet)
 	 * Returns all units within @c radius of @c pos,
 	 * and treats each unit as a 3D point object
 	 */
-	std::vector<CUnit*> GetUnits(const float3& pos, float radius);
+	const std::vector<CUnit*>& GetUnits(const float3& pos, float radius);
 	/**
 	 * Returns all units within @c radius of @c pos,
 	 * takes the 3D model radius of each unit into account,
  	 * and performs the search within a sphere or cylinder depending on @c spherical
 	 */
-	std::vector<CUnit*> GetUnitsExact(const float3& pos, float radius, bool spherical = true);
+	const std::vector<CUnit*>& GetUnitsExact(const float3& pos, float radius, bool spherical = true);
 	/**
 	 * Returns all units within the rectangle defined by
 	 * mins and maxs, which extends infinitely along the y-axis
 	 */
-	std::vector<CUnit*> GetUnitsExact(const float3& mins, const float3& maxs);
+	const std::vector<CUnit*>& GetUnitsExact(const float3& mins, const float3& maxs);
 	/**
 	 * Returns all features within @c radius of @c pos,
 	 * takes the 3D model radius of each feature into account,
 	 * and performs the search within a sphere or cylinder depending on @c spherical
 	 */
-	std::vector<CFeature*> GetFeaturesExact(const float3& pos, float radius, bool spherical = true);
+	const std::vector<CFeature*>& GetFeaturesExact(const float3& pos, float radius, bool spherical = true);
 	/**
 	 * Returns all features within the rectangle defined by
 	 * mins and maxs, which extends infinitely along the y-axis
 	 */
-	std::vector<CFeature*> GetFeaturesExact(const float3& mins, const float3& maxs);
+	const std::vector<CFeature*>& GetFeaturesExact(const float3& mins, const float3& maxs);
 
-	std::vector<CProjectile*> GetProjectilesExact(const float3& pos, float radius);
-	std::vector<CProjectile*> GetProjectilesExact(const float3& mins, const float3& maxs);
+	const std::vector<CProjectile*>& GetProjectilesExact(const float3& pos, float radius);
+	const std::vector<CProjectile*>& GetProjectilesExact(const float3& mins, const float3& maxs);
 
-	std::vector<CSolidObject*> GetSolidsExact(
+	const std::vector<CSolidObject*>& GetSolidsExact(
+		const float3& pos,
+		const float radius,
+		const unsigned int physicalStateBits = 0xFFFFFFFF,
+		const unsigned int collisionStateBits = 0xFFFFFFFF
+	);
+
+	bool NoSolidsExact(
 		const float3& pos,
 		const float radius,
 		const unsigned int physicalStateBits = 0xFFFFFFFF,
@@ -136,7 +143,6 @@ private:
 	// numQuadsX * numQuadsZ (eg. tempQuads) -- GetQuadsOnRay ensures
 	// this by itself, for GetQuads the callers take care of it
 	//
-	void GetQuads(float3 pos, float radius, std::vector<int>* quads) const;
 
 	int2 WorldPosToQuadField(const float3 p) const;
 	int WorldPosToQuadFieldIdx(const float3 p) const;
@@ -144,6 +150,13 @@ private:
 private:
 	std::vector<Quad> baseQuads;
 
+	// preallocated vectors for Get*Exact functions
+	std::vector<CUnit*> tempUnits;
+	std::vector<CFeature*> tempFeatures;
+	std::vector<CProjectile*> tempProjectiles;
+	std::vector<CSolidObject*> tempSolids;
+	std::vector<int> tempQuads;
+
 	int numQuadsX;
 	int numQuadsZ;
 
