@@ -50,10 +50,6 @@ CResourceMapAnalyzer::CResourceMapAnalyzer(int resourceId)
 	, minIncomeForSpot(0)
 	, xtractorRadius(0)
 	, doubleRadius(0)
-	, rexArrayA(NULL)
-	, rexArrayB(NULL)
-	, rexArrayC(NULL)
-	, tempAverage(NULL)
 {
 	if (CACHE_BASE.empty()) {
 		CACHE_BASE = dataDirsAccess.LocateDir(FileSystem::GetCacheDir() + "/analyzedResourceMaps/", FileQueryFlags::WRITE | FileQueryFlags::CREATE_DIRS);
@@ -79,24 +75,16 @@ CResourceMapAnalyzer::CResourceMapAnalyzer(int resourceId)
 	squareRadius = xtractorRadius * xtractorRadius;
 	doubleSquareRadius = doubleRadius * doubleRadius;
 
-	rexArrayA = new unsigned char[totalCells];
-	rexArrayB = new unsigned char[totalCells];
+	rexArrayA.resize(totalCells);
+	rexArrayB.resize(totalCells);
 	// used for drawing the TGA, not really needed with a couple of changes
-	rexArrayC = new unsigned char[totalCells];
+	rexArrayC.resize(totalCells);
 
-	tempAverage = new int[totalCells];
+	tempAverage.resize(totalCells);
 
 	Init();
 }
 
-CResourceMapAnalyzer::~CResourceMapAnalyzer() {
-
-	delete[] rexArrayA;
-	delete[] rexArrayB;
-	delete[] rexArrayC;
-	delete[] tempAverage;
-}
-
 
 
 float3 CResourceMapAnalyzer::GetNearestSpot(int builderUnitId, const UnitDef* extractor) const {
@@ -174,7 +162,7 @@ const std::vector<float3>& CResourceMapAnalyzer::GetSpots() const {
 }
 
 void CResourceMapAnalyzer::GetResourcePoints() {
-	int* xend = new int[doubleRadius + 1];
+	std::vector<int> xend(doubleRadius + 1);
 
 	for (int a = 0; a < doubleRadius + 1; a++) {
 		float z = a - xtractorRadius;
@@ -199,7 +187,6 @@ void CResourceMapAnalyzer::GetResourcePoints() {
 	if (totalResourcesDouble < 0.9) {
 		// the map does not have any resource, just stop
 		numSpotsFound = 0;
-		delete[] xend;
 		return;
 	}
 
@@ -280,12 +267,7 @@ void CResourceMapAnalyzer::GetResourcePoints() {
 	}
 
 	// make a list for the distribution of values
-	int* valueDist = new int[256];
-
-	for (int i = 0; i < 256; i++) {
-		// clear the array (useless?)
-		valueDist[i] = 0;
-	}
+	std::vector<int> valueDist(256, 0);
 
 	// this will get the total resources a rex placed at each spot would make
 	for (int i = 0; i < totalCells; i++) {
@@ -318,7 +300,7 @@ void CResourceMapAnalyzer::GetResourcePoints() {
 		numberOfValues = 256;
 	}
 
-	int* bestSpotList = new int[numberOfValues];
+	std::vector<int> bestSpotList(numberOfValues);
 
 	for (int i = 0; i < totalCells; i++) {
 		if (rexArrayB[i] == bestValue) {
@@ -379,8 +361,8 @@ void CResourceMapAnalyzer::GetResourcePoints() {
 						numberOfValues = 256;
 					}
 
-					delete[] bestSpotList;
-					bestSpotList = new int[numberOfValues];
+					bestSpotList.clear();
+					bestSpotList.resize(numberOfValues);
 
 					for (int i = 0; i < totalCells; i++) {
 						if (rexArrayB[i] == bestValue) {
@@ -535,11 +517,6 @@ void CResourceMapAnalyzer::GetResourcePoints() {
 		}
 	}
 
-	// kill the lists
-	delete[] bestSpotList;
-	delete[] valueDist;
-	delete[] xend;
-
 	// 0.95 used for reliability
 	// bool isResourceMap = (numSpotsFound > maxSpots * 0.95);
 }
