@@ -3,6 +3,8 @@
 #include "SimObjectIDPool.h"
 #include "GlobalSynced.h"
 #include "Sim/Objects/SolidObject.h"
+#include "System/creg/STL_Map.h"
+
 
 CR_BIND(SimObjectIDPool, )
 CR_REG_METADATA(SimObjectIDPool, (
@@ -20,9 +22,8 @@ void SimObjectIDPool::Expand(unsigned int baseID, unsigned int numIDs) {
 	}
 
 	// randomize so that Lua widgets can not easily determine counts
-	SyncedRNG rng;
-	std::random_shuffle(newIDs.begin(), newIDs.end(), rng);
-	std::random_shuffle(newIDs.begin(), newIDs.end(), rng);
+	std::random_shuffle(newIDs.begin(), newIDs.end(), gs->rng);
+	std::random_shuffle(newIDs.begin(), newIDs.end(), gs->rng);
 
 	// NOTE:
 	//   any randomization would be undone by using a std::set as-is
