@@ -42,8 +42,7 @@ protected: // IPathFinder impl
 		const CSolidObject* owner,
 		const unsigned int pathOptDir,
 		const unsigned int blockStatus,
-		float speedMod,
-		bool withinConstraints
+		float speedMod
 	);
 	/**
 	 * Recreates the path found by pathfinder.
