@@ -15,22 +15,34 @@ void IPathController::FreeInstance(IPathController* instance) {
 
 float GMTDefaultPathController::GetDeltaSpeed(
 	unsigned int pathID,
-	float targetSpeed,
-	float currentSpeed,
-	float maxAccRate,
-	float maxDecRate,
+	float targetSpeed, // unsigned
+	float currentSpeed, // unsigned
+	float maxAccRate, // unsigned
+	float maxDecRate, // unsigned
 	bool wantReverse,
 	bool isReversing
 ) const {
-	const int targetSpeedSign = Sign(int(!wantReverse));
-	const int currentSpeedSign = Sign(int(!isReversing));
+	// Sign(0) is negative which we do not want
+	const int  targetSpeedSign = Sign(int(!wantReverse) * 2 - 1);
+	const int currentSpeedSign = Sign(int(!isReversing) * 2 - 1);
 
-	const float rawSpeedDiff = (targetSpeed * targetSpeedSign) - (currentSpeed * currentSpeedSign);
-	const float absSpeedDiff = math::fabs(rawSpeedDiff);
-	// need to clamp, game-supplied values can be much larger than |speedDiff|
+	// if reversing, target and current are swapped
+	const float speeds[] = {targetSpeed * targetSpeedSign, currentSpeed * currentSpeedSign};
+
+	// all possible cases
+	//   tar=[6,+1] && cur=[4,+1] --> dif=+2, delta= accRate*+1 (fwd-acc from +4 to +6)
+	//   tar=[4,+1] && cur=[6,+1] --> dif=-2, delta=-decRate*+1 (fwd-dec from +6 to +4)
+	//   tar=[6,-1] && cur=[4,-1] --> dif=+2, delta= accRate*-1 (rev-acc from -4 to -6)
+	//   tar=[4,-1] && cur=[6,-1] --> dif=-2, delta=-decRate*-1 (rev-dec from -6 to -4)
+	//   tar=[4,-1] && cur=[4,+1] --> dif=-8, delta=-decRate*+1 (fwd-dec from +5 to 0, rev-acc from 0 to -5)
+	//   tar=[4,+1] && cur=[4,-1] --> dif=-8, delta=-decRate*-1 (rev-dec from -5 to 0, fwd-acc from 0 to +5)
+	//
+	const float rawSpeedDiff = speeds[isReversing] - speeds[1 - isReversing];
+	const float absSpeedDiff = std::max(rawSpeedDiff, -rawSpeedDiff);
+
+	// need to clamp, rates can be much larger than |speedDiff|
 	const float modAccRate = std::min(absSpeedDiff, maxAccRate);
 	const float modDecRate = std::min(absSpeedDiff, maxDecRate);
-
 	const float deltaSpeed = mix(modAccRate, -modDecRate, (rawSpeedDiff < 0.0f));
 
 	// no acceleration changes if not on ground
@@ -39,7 +51,7 @@ float GMTDefaultPathController::GetDeltaSpeed(
 	// ships should always use (1 - IsInWater()), hovercraft
 	// should use (1 - IsInWater()) but only when over water,
 	// tanks should also test if IsInWater() && !IsOnGround()
-	return (deltaSpeed * (1 - owner->IsInAir()));
+	return ((deltaSpeed * currentSpeedSign) * (1 - owner->IsInAir()));
 }
 
 #if 1
