@@ -17,7 +17,7 @@
 #include "Sim/Units/Unit.h"
 #include "System/creg/STL_List.h"
 
-CR_BIND_DERIVED(CFireProjectile, CProjectile, (ZeroVector,ZeroVector,NULL,0,0,0,0))
+CR_BIND_DERIVED(CFireProjectile, CProjectile, )
 CR_BIND(CFireProjectile::SubParticle, )
 
 CR_REG_METADATA(CFireProjectile,(
@@ -101,9 +101,9 @@ void CFireProjectile::Update()
 			subParticles2.push_front(sub);
 		}
 		if (!(ttl & 31)) {
-			// synced code
-			const std::vector<CFeature*>& features = quadField->GetFeaturesExact(emitPos + wind.GetCurrentWind() * 0.7f, emitRadius * 2);
-			const std::vector<CUnit*>& units = quadField->GetUnitsExact(emitPos + wind.GetCurrentWind() * 0.7f, emitRadius * 2);
+			// copy on purpose, since the below can call Lua
+			const std::vector<CFeature*> features = quadField->GetFeaturesExact(emitPos + wind.GetCurrentWind() * 0.7f, emitRadius * 2);
+			const std::vector<CUnit*> units = quadField->GetUnitsExact(emitPos + wind.GetCurrentWind() * 0.7f, emitRadius * 2);
 
 			for (CFeature* f: features) {
 				if (gs->randFloat() > 0.8f) {
