@@ -12,7 +12,7 @@
 #include "Sim/Units/UnitHandler.h"
 #include "System/Matrix44f.h"
 
-CR_BIND_DERIVED(CProjectile, CExpGenSpawnable, )
+CR_BIND_DERIVED_INTERFACE(CProjectile, CExpGenSpawnable)
 
 CR_REG_METADATA(CProjectile,
 (
