@@ -11,7 +11,7 @@
 #include "Sim/Weapons/WeaponDef.h"
 #include "System/Sync/SyncTracer.h"
 
-CR_BIND_DERIVED(CEmgProjectile, CWeaponProjectile, (ProjectileParams()))
+CR_BIND_DERIVED(CEmgProjectile, CWeaponProjectile, )
 
 CR_REG_METADATA(CEmgProjectile,(
 	CR_SETFLAG(CF_Synced),
@@ -19,6 +19,7 @@ CR_REG_METADATA(CEmgProjectile,(
 	CR_MEMBER(color)
 ))
 
+
 CEmgProjectile::CEmgProjectile(const ProjectileParams& params): CWeaponProjectile(params)
 {
 	projectileType = WEAPON_EMG_PROJECTILE;
