@@ -24,16 +24,7 @@ CR_BIND_DERIVED(CFactoryCAI ,CCommandAI , )
 
 CR_REG_METADATA(CFactoryCAI , (
 	CR_MEMBER(newUnitCommands),
-	CR_MEMBER(buildOptions),
-	CR_POSTLOAD(PostLoad)
-))
-
-CR_BIND(CFactoryCAI::BuildOption, )
-
-CR_REG_METADATA_SUB(CFactoryCAI,BuildOption , (
-	CR_MEMBER(name),
-	CR_MEMBER(fullName),
-	CR_MEMBER(numQued)
+	CR_MEMBER(buildOptions)
 ))
 
 static std::string GetUnitDefBuildOptionToolTip(const UnitDef* ud, bool disabled) {
@@ -66,8 +57,7 @@ CFactoryCAI::CFactoryCAI(CUnit* owner): CCommandAI(owner)
 	newUnitCommands.SetQueueType(CCommandQueue::NewUnitQueueType);
 
 	if (owner->unitDef->canmove) {
-		possibleCommands.emplace_back();
-		CommandDescription& c = possibleCommands.back();
+		SCommandDescription c;
 
 		c.id        = CMD_MOVE;
 		c.type      = CMDTYPE_ICON_MAP;
@@ -76,11 +66,11 @@ CFactoryCAI::CFactoryCAI(CUnit* owner): CCommandAI(owner)
 		c.name      = "Move";
 		c.tooltip   = c.name + ": Order ready built units to move to a position";
 		c.mouseicon = c.name;
+		possibleCommands.push_back(commandDescriptionCache->GetPtr(c));
 	}
 
 	if (owner->unitDef->canPatrol) {
-		possibleCommands.emplace_back();
-		CommandDescription& c = possibleCommands.back();
+		SCommandDescription c;
 
 		c.id        = CMD_PATROL;
 		c.type      = CMDTYPE_ICON_MAP;
@@ -89,11 +79,11 @@ CFactoryCAI::CFactoryCAI(CUnit* owner): CCommandAI(owner)
 		c.name      = "Patrol";
 		c.tooltip   = c.name + ": Order ready built units to patrol to one or more waypoints";
 		c.mouseicon = c.name;
+		possibleCommands.push_back(commandDescriptionCache->GetPtr(c));
 	}
 
 	if (owner->unitDef->canFight) {
-		possibleCommands.emplace_back();
-		CommandDescription& c = possibleCommands.back();
+		SCommandDescription c;
 
 		c.id        = CMD_FIGHT;
 		c.type      = CMDTYPE_ICON_MAP;
@@ -102,11 +92,11 @@ CFactoryCAI::CFactoryCAI(CUnit* owner): CCommandAI(owner)
 		c.name      = "Fight";
 		c.tooltip   = c.name + ": Order ready built units to take action while moving to a position";
 		c.mouseicon = c.name;
+		possibleCommands.push_back(commandDescriptionCache->GetPtr(c));
 	}
 
 	if (owner->unitDef->canGuard) {
-		possibleCommands.emplace_back();
-		CommandDescription& c = possibleCommands.back();
+		SCommandDescription c;
 
 		c.id        = CMD_GUARD;
 		c.type      = CMDTYPE_ICON_UNIT;
@@ -115,6 +105,7 @@ CFactoryCAI::CFactoryCAI(CUnit* owner): CCommandAI(owner)
 		c.name      = "Guard";
 		c.tooltip   = c.name + ": Order ready built units to guard another unit and attack units attacking it";
 		c.mouseicon = c.name;
+		possibleCommands.push_back(commandDescriptionCache->GetPtr(c));
 	}
 
 	CFactory* fac = static_cast<CFactory*>(owner);
@@ -132,8 +123,7 @@ CFactoryCAI::CFactoryCAI(CUnit* owner): CCommandAI(owner)
 		}
 
 		{
-			possibleCommands.emplace_back();
-			CommandDescription& c = possibleCommands.back();
+			SCommandDescription c;
 
 			c.id   = -ud->id; // build-options are always negative
 			c.type = CMDTYPE_ICON;
@@ -143,13 +133,8 @@ CFactoryCAI::CFactoryCAI(CUnit* owner): CCommandAI(owner)
 			c.mouseicon = c.name;
 			c.tooltip   = GetUnitDefBuildOptionToolTip(ud, c.disabled = (ud->maxThisUnit <= 0));
 
-
-			BuildOption bo;
-			bo.name     = name;
-			bo.fullName = name;
-			bo.numQued  = 0;
-
-			buildOptions[c.id] = bo;
+			buildOptions[c.id] = 0;
+			possibleCommands.push_back(commandDescriptionCache->GetPtr(c));
 		}
 	}
 }
@@ -239,15 +224,15 @@ void CFactoryCAI::GiveCommandReal(const Command& c, bool fromSynced)
 		return;
 	}
 
-	BuildOption& bo = boi->second;
+	int& numQueued = boi->second;
 	int numItems = 1;
 
 	if (c.options & SHIFT_KEY)   { numItems *= 5; }
 	if (c.options & CONTROL_KEY) { numItems *= 20; }
 
 	if (c.options & RIGHT_MOUSE_KEY) {
-		bo.numQued -= numItems;
-		bo.numQued  = std::max(bo.numQued, 0);
+		numQueued -= numItems;
+		numQueued  = std::max(numQueued, 0);
 
 		int numToErase = numItems;
 		if (c.options & ALT_KEY) {
@@ -265,7 +250,7 @@ void CFactoryCAI::GiveCommandReal(const Command& c, bool fromSynced)
 				}
 			}
 		}
-		UpdateIconName(cmdID, bo);
+		UpdateIconName(cmdID, numQueued);
 		SlowUpdate();
 	} else {
 		if (c.options & ALT_KEY) {
@@ -291,8 +276,8 @@ void CFactoryCAI::GiveCommandReal(const Command& c, bool fromSynced)
 				commandQue.push_back(c);
 			}
 		}
-		bo.numQued += numItems;
-		UpdateIconName(cmdID, bo);
+		numQueued += numItems;
+		UpdateIconName(cmdID, numQueued);
 
 		SlowUpdate();
 	}
@@ -302,9 +287,9 @@ void CFactoryCAI::GiveCommandReal(const Command& c, bool fromSynced)
 void CFactoryCAI::InsertBuildCommand(CCommandQueue::iterator& it,
                                      const Command& newCmd)
 {
-	map<int, BuildOption>::iterator boi = buildOptions.find(newCmd.GetID());
+	const auto boi = buildOptions.find(newCmd.GetID());
 	if (boi != buildOptions.end()) {
-		boi->second.numQued++;
+		boi->second++;
 		UpdateIconName(newCmd.GetID(), boi->second);
 	}
 	if (!commandQue.empty() && (it == commandQue.begin())) {
@@ -319,9 +304,9 @@ void CFactoryCAI::InsertBuildCommand(CCommandQueue::iterator& it,
 bool CFactoryCAI::RemoveBuildCommand(CCommandQueue::iterator& it)
 {
 	Command& cmd = *it;
-	map<int, BuildOption>::iterator boi = buildOptions.find(cmd.GetID());
+	const auto boi = buildOptions.find(cmd.GetID());
 	if (boi != buildOptions.end()) {
-		boi->second.numQued--;
+		boi->second--;
 		UpdateIconName(cmd.GetID(), boi->second);
 	}
 	if (!commandQue.empty() && (it == commandQue.begin())) {
@@ -338,16 +323,16 @@ bool CFactoryCAI::RemoveBuildCommand(CCommandQueue::iterator& it)
 }
 
 
-void CFactoryCAI::DecreaseQueueCount(const Command& buildCommand, BuildOption& buildOption)
+void CFactoryCAI::DecreaseQueueCount(const Command& buildCommand, int& numQueued)
 {
 	// copy in case we get pop'ed
 	// NOTE: the queue should not be empty at this point!
 	const Command frontCommand = commandQue.empty()? Command(CMD_STOP): commandQue.front();
 
 	if (!repeatOrders || (buildCommand.options & INTERNAL_ORDER))
-		buildOption.numQued--;
+		numQueued--;
 
-	UpdateIconName(buildCommand.GetID(), buildOption);
+	UpdateIconName(buildCommand.GetID(), numQueued);
 
 	// if true, factory was set to wait and its buildee
 	// could only have been finished by assisting units
@@ -371,9 +356,7 @@ void CFactoryCAI::DecreaseQueueCount(const Command& buildCommand, BuildOption& b
 //   only called if Factory::QueueBuild returned FACTORY_NEXT_BUILD_ORDER
 //   (meaning the order was not rejected and the callback was installed)
 void CFactoryCAI::FactoryFinishBuild(const Command& command) {
-	CFactoryCAI::BuildOption& bo = buildOptions[command.GetID()];
-
-	DecreaseQueueCount(command, bo);
+	DecreaseQueueCount(command, buildOptions[command.GetID()]);
 }
 
 void CFactoryCAI::SlowUpdate()
@@ -390,9 +373,8 @@ void CFactoryCAI::SlowUpdate()
 		Command& c = commandQue.front();
 
 		const size_t oldQueueSize = commandQue.size();
-		const std::map<int, BuildOption>::iterator buildOptIt = buildOptions.find(c.GetID());
 
-		if (buildOptIt != buildOptions.end()) {
+		if (buildOptions.find(c.GetID()) != buildOptions.end()) {
 			// build-order
 			switch (fac->QueueBuild(unitDefHandler->GetUnitDefByID(-c.GetID()), c)) {
 				case CFactory::FACTORY_SKIP_BUILD_ORDER: {
@@ -442,22 +424,24 @@ int CFactoryCAI::GetDefaultCmd(const CUnit* pointed, const CFeature* feature)
 }
 
 
-void CFactoryCAI::UpdateIconName(int cmdID, const BuildOption& bo)
+void CFactoryCAI::UpdateIconName(int cmdID, const int& numQueued)
 {
-	vector<CommandDescription>::iterator pci;
-	for (pci = possibleCommands.begin(); pci != possibleCommands.end(); ++pci) {
-		if (pci->id != cmdID)
+	for (auto pci = possibleCommands.begin(); pci != possibleCommands.end(); ++pci) {
+		const SCommandDescription* cd = *pci;
+		if (cd->id != cmdID)
 			continue;
 
 		char t[32];
-		SNPRINTF(t, 10, "%d", bo.numQued);
+		SNPRINTF(t, 10, "%d", numQueued);
 
-		pci->name = bo.name;
-		pci->params.clear();
+		SCommandDescription c = *cd;
+		c.params.clear();
 
-		if (bo.numQued)
-			pci->params.push_back(t);
+		if (numQueued > 0)
+			c.params.push_back(t);
 
+		commandDescriptionCache->DecRef(*cd);
+		*pci = commandDescriptionCache->GetPtr(c);
 		break;
 	}
 
