@@ -6,7 +6,6 @@
 #include "LuaUnitScript.h"
 
 #include "CobInstance.h"
-#include "UnitScriptLog.h"
 #include "LuaInclude.h"
 #include "NullUnitScript.h"
 #include "LuaScriptNames.h"
@@ -188,7 +187,7 @@ CUnitScript* CLuaUnitScript::activeScript;
 
 
 CLuaUnitScript::CLuaUnitScript(lua_State* L, CUnit* unit)
-	: CUnitScript(unit)
+	: CNullUnitScript(unit)
 	, handle(CLuaHandle::GetHandle(L)), L(L)
 	, scriptIndex(LUAFN_Last, LUA_NOREF)
 	, inKilled(false)
@@ -299,7 +298,7 @@ void CLuaUnitScript::UpdateCallIn(const string& fname, int ref)
 		case LUAFN_StartBuilding: hasStartBuilding = (ref != LUA_NOREF); break;
 	}
 
-	LUA_TRACE(fname.c_str());
+	//LUA_TRACE(fname.c_str());
 }
 
 
@@ -313,7 +312,7 @@ void CLuaUnitScript::RemoveCallIn(const string& fname)
 		UpdateCallIn(fname, LUA_NOREF);
 	}
 
-	LUA_TRACE(fname.c_str());
+	//LUA_TRACE(fname.c_str());
 }
 
 
@@ -426,15 +425,15 @@ int CLuaUnitScript::RunQueryCallIn(int fn)
 
 	const int scriptPieceNum = (int)PopNumber(fn, 0) - 1;
 
-	if (LOG_IS_ENABLED(L_DEBUG)) {
-		if (PieceExists(scriptPieceNum)) {
-			LocalModelPiece* piece = GetScriptLocalModelPiece(scriptPieceNum);
-			LOG_L(L_DEBUG, "%s: %d %s",
-					CLuaUnitScriptNames::GetScriptName(fn).c_str(),
-					scriptPieceNum,
-					(piece->original) ? piece->original->name.c_str() : "n/a");
-		}
-	}
+	// if (LOG_IS_ENABLED(L_DEBUG)) {
+		// if (PieceExists(scriptPieceNum)) {
+			// LocalModelPiece* piece = GetScriptLocalModelPiece(scriptPieceNum);
+			// LOG_L(L_DEBUG, "%s: %d %s",
+					// CLuaUnitScriptNames::GetScriptName(fn).c_str(),
+					// scriptPieceNum,
+					// (piece->original) ? piece->original->name.c_str() : "n/a");
+		// }
+	// }
 
 	return scriptPieceNum;
 }
@@ -456,15 +455,15 @@ int CLuaUnitScript::RunQueryCallIn(int fn, float arg1)
 
 	const int scriptPieceNum = (int)PopNumber(fn, 0) - 1;
 
-	if (LOG_IS_ENABLED(L_DEBUG)) {
-		if (PieceExists(scriptPieceNum)) {
-			LocalModelPiece* piece = GetScriptLocalModelPiece(scriptPieceNum);
-			LOG_L(L_DEBUG, "%s: %d %s",
-					CLuaUnitScriptNames::GetScriptName(fn).c_str(),
-					scriptPieceNum,
-					(piece->original) ? piece->original->name.c_str() : "n/a");
-		}
-	}
+	// if (LOG_IS_ENABLED(L_DEBUG)) {
+		// if (PieceExists(scriptPieceNum)) {
+			// LocalModelPiece* piece = GetScriptLocalModelPiece(scriptPieceNum);
+			// LOG_L(L_DEBUG, "%s: %d %s",
+					// CLuaUnitScriptNames::GetScriptName(fn).c_str(),
+					// scriptPieceNum,
+					// (piece->original) ? piece->original->name.c_str() : "n/a");
+		// }
+	// }
 
 	return scriptPieceNum;
 }
@@ -1009,7 +1008,7 @@ int CLuaUnitScript::CreateScript(lua_State* L)
 {
 	CUnit* unit = ParseUnit(L, __FUNCTION__, 1);
 	if (unit == NULL) {
-		LUA_TRACE("no such unit");
+		//LUA_TRACE("no such unit");
 		return 0;
 	}
 
@@ -1037,7 +1036,7 @@ int CLuaUnitScript::CreateScript(lua_State* L)
 		w->SetWeaponNum(w->weaponNum);
 	}
 
-	LUA_TRACE("script replaced with CLuaUnitScript");
+	//LUA_TRACE("script replaced with CLuaUnitScript");
 
 	return 0;
 }
@@ -1329,18 +1328,6 @@ int CLuaUnitScript::Turn(lua_State* L)
 		activeScript->Turn(piece, axis, speed, dest);
 	}
 
-	if (lua_isboolean(L, 5) && lua_toboolean(L, 5)) {
-		// CUnit* unit = activeScript->GetUnit();
-		// LocalModel* model = unit->localModel;
-		LocalModelPiece* piece = ParseLocalModelPiece(L, activeScript, __FUNCTION__);
-
-		// note:
-		//   both of these only have effect if MoveNow() was called, but
-		//   the former starts from the ROOT piece (less efficient here)
-		// model->UpdatePieceMatrices();
-		piece->UpdateMatricesRec(true);
-	}
-
 	return 0;
 }
 
@@ -1363,18 +1350,6 @@ int CLuaUnitScript::Move(lua_State* L)
 		activeScript->Move(piece, axis, speed, dest);
 	}
 
-	if (lua_isboolean(L, 5) && lua_toboolean(L, 5)) {
-		// CUnit* unit = activeScript->GetUnit();
-		// LocalModel* model = unit->localModel;
-		LocalModelPiece* piece = ParseLocalModelPiece(L, activeScript, __FUNCTION__);
-
-		// note:
-		//   both of these only have effect if MoveNow() was called, but
-		//   the former starts from the ROOT piece (less efficient here)
-		// model->UpdatePieceMatrices();
-		piece->UpdateMatricesRec(true);
-	}
-
 	return 0;
 }
 
@@ -1424,7 +1399,7 @@ int CLuaUnitScript::WaitForAnimation(lua_State* L, const char* caller, AnimType
 	const int piece = luaL_checkint(L, 1) - 1;
 	const int axis  = ParseAxis(L, caller, 2);
 
-	lua_pushboolean(L, script->AddAnimListener(type, piece, axis, script));
+	lua_pushboolean(L, script->NeedsWait(type, piece, axis));
 	return 1;
 }
 
