@@ -5,7 +5,8 @@
 #ifndef UNIT_SCRIPT_ENGINE_H
 #define UNIT_SCRIPT_ENGINE_H
 
-#include <list>
+#include "System/creg/creg_cond.h"
+#include <vector>
 
 class CUnit;
 class CUnitScript;
@@ -13,9 +14,11 @@ class CUnitScript;
 
 class CUnitScriptEngine
 {
+	CR_DECLARE_STRUCT(CUnitScriptEngine)
+
 protected:
-	std::list<CUnitScript*> animating; // hash would be optimal, but not crucial
-	void CheckForDuplicates(const char* name, CUnitScript* instance);
+	std::vector<CUnitScript*> animating; // hash would be optimal, but not crucial
+	void CheckForDuplicates(const char* name, const CUnitScript* instance);
 
 public:
 	CUnitScriptEngine();
@@ -24,10 +27,12 @@ public:
 	void RemoveInstance(CUnitScript* instance);
 	void Tick(int deltaTime);
 
+	static void InitStatic();
+	static void KillStatic();
 private:
 	CUnitScript* currentScript;
 };
 
-extern CUnitScriptEngine GUnitScriptEngine;
+extern CUnitScriptEngine* unitScriptEngine;
 
 #endif // UNIT_SCRIPT_ENGINE_H
