@@ -14,7 +14,7 @@ public:
 	void UpdateWantedDir() override final;
 
 private:
-	bool HaveFreeLineOfFire(const float3 pos, const SWeaponTarget& trg) const override final;
+	bool HaveFreeLineOfFire(const float3 pos, const SWeaponTarget& trg, bool useMuzzle = false) const override final;
 	void FireImpl(const bool scriptCall) override final;
 };
 
