@@ -24,6 +24,7 @@
 #include "Sim/Weapons/NoWeapon.h"
 #include "System/EventHandler.h"
 #include "System/myMath.h"
+#include "System/creg/DefTypes.h"
 #include "System/Sync/SyncTracer.h"
 #include "System/Sound/ISoundChannels.h"
 #include "System/Log/ILog.h"
@@ -32,8 +33,7 @@ CR_BIND_DERIVED(CWeapon, CObject, (NULL, NULL))
 
 CR_REG_METADATA(CWeapon, (
 	CR_MEMBER(owner),
-	CR_MEMBER(weaponDefID),
-	CR_IGNORED(weaponDef), //retrieved by id
+	CR_MEMBER(weaponDef),
 	CR_MEMBER(aimFromPiece),
 	CR_MEMBER(muzzlePiece),
 	CR_MEMBER(range),
@@ -92,9 +92,7 @@ CR_REG_METADATA(CWeapon, (
 	CR_MEMBER(errorVectorAdd),
 
 	CR_MEMBER(currentTarget),
-	CR_MEMBER(currentTargetPos),
-
-	CR_POSTLOAD(PostLoad)
+	CR_MEMBER(currentTargetPos)
 ))
 
 
@@ -407,7 +405,7 @@ void CWeapon::UpdateFire()
 	if (!CanFire(false, false, false))
 		return;
 
-	if (!TryTarget(currentTargetPos, currentTarget))
+	if (!TryTarget(currentTargetPos, currentTarget, true))
 		return;
 
 	// pre-check if we got enough resources (so CobBlockShot gets only called when really possible to shoot)
@@ -842,7 +840,7 @@ float3 CWeapon::GetTargetBorderPos(
 }
 
 
-bool CWeapon::TryTarget(const float3 tgtPos, const SWeaponTarget& trg) const
+bool CWeapon::TryTarget(const float3 tgtPos, const SWeaponTarget& trg, bool preFire) const
 {
 	assert(GetLeadTargetPos(trg).SqDistance(tgtPos) < Square(250.f));
 
@@ -852,11 +850,11 @@ bool CWeapon::TryTarget(const float3 tgtPos, const SWeaponTarget& trg) const
 		return false;
 
 	// no LOF if aim-position is below ground (not in HFLOF, is overridden)
-	if (aimFromPos.y < CGround::GetHeightReal(aimFromPos.x, aimFromPos.z))
+	if (preFire && (weaponMuzzlePos.y < CGround::GetHeightReal(weaponMuzzlePos.x, weaponMuzzlePos.z)))
 		return false;
 
 	//FIXME add a forcedUserTarget (a forced fire mode enabled with ctrl key or something) and skip the tests below then
-	return HaveFreeLineOfFire(tgtPos, trg);
+	return HaveFreeLineOfFire(tgtPos, trg, preFire);
 }
 
 
@@ -954,9 +952,10 @@ bool CWeapon::TestRange(const float3 tgtPos, const SWeaponTarget& trg) const
 }
 
 
-bool CWeapon::HaveFreeLineOfFire(const float3 pos, const SWeaponTarget& trg) const
+bool CWeapon::HaveFreeLineOfFire(const float3 pos, const SWeaponTarget& trg, bool useMuzzle) const
 {
-	float3 dir = pos - aimFromPos;
+	float3 testPos = useMuzzle ? weaponMuzzlePos : aimFromPos;
+	float3 dir = pos - testPos;
 
 	const float length = dir.LengthNormalize();
 	const float spread = AccuracyExperience() + SprayAngleExperience();
@@ -972,8 +971,8 @@ bool CWeapon::HaveFreeLineOfFire(const float3 pos, const SWeaponTarget& trg) con
 		CUnit* unit = nullptr;
 		CFeature* feature = nullptr;
 
-		const float gdst = TraceRay::TraceRay(aimFromPos, dir, length, ~Collision::NOGROUND, owner, unit, feature);
-		const float3 gpos = aimFromPos + dir * gdst;
+		const float gdst = TraceRay::TraceRay(testPos, dir, length, ~Collision::NOGROUND, owner, unit, feature);
+		const float3 gpos = testPos + dir * gdst;
 
 		// true iff ground does not block the ray of length <length> from <pos> along <dir>
 		if ((gdst > 0.0f) && (gpos.SqDistance(pos) > Square(damages->damageAreaOfEffect)))
@@ -981,9 +980,8 @@ bool CWeapon::HaveFreeLineOfFire(const float3 pos, const SWeaponTarget& trg) con
 	}
 
 	// friendly, neutral & feature check
-	if (TraceRay::TestCone(aimFromPos, dir, length, spread, owner->allyteam, avoidFlags, owner)) {
+	if (TraceRay::TestCone(testPos, dir, length, spread, owner->allyteam, avoidFlags, owner))
 		return false;
-	}
 
 	return true;
 }
@@ -1251,8 +1249,3 @@ float3 CWeapon::GetLeadTargetPos(const SWeaponTarget& target) const
 
 	return currentTargetPos;
 }
-
-void CWeapon::PostLoad()
-{
-	weaponDef = weaponDefHandler->GetWeaponDefByID(weaponDefID);
-}
