@@ -266,6 +266,8 @@ WeaponDef::WeaponDef()
 	noAutoTarget = false;
 	onlyForward = false;
 
+	damages.fromDef = true;
+
 	const LuaTable wdTable;
 	WeaponDefs.Load(this, wdTable);
 }
@@ -350,6 +352,7 @@ WeaponDef::WeaponDef(const LuaTable& wdTable, const std::string& name_, int id_)
 			defDamage = 1.0f;
 
 		damages.SetDefaultDamage(defDamage);
+		damages.fromDef = true;
 
 		if (!paralyzer)
 			damages.paralyzeDamageTime = 0;
@@ -586,7 +589,7 @@ S3DModel* WeaponDef::LoadModel()
 {
 	if (visuals.model == NULL) {
 		if (!visuals.modelName.empty()) {
-			visuals.model = modelParser->Load3DModel(visuals.modelName);
+			visuals.model = modelLoader.LoadModel(visuals.modelName);
 		} else {
 			// not useful, too much spam
 			// LOG_L(L_WARNING, "[WeaponDef::%s] weapon \"%s\" has no model defined", __FUNCTION__, name.c_str());
