@@ -3,11 +3,10 @@
 #ifndef CONFIG_VALUE_H
 #define CONFIG_VALUE_H
 
-#include <boost/utility.hpp>
+#include <boost/noncopyable.hpp>
 #include <map>
-#include <sstream>
 #include <string>
-#include "System/Util.h"
+#include "System/StringConvertibleOptionalValue.h"
 
 
 
