@@ -23,9 +23,13 @@
 #include "System/Util.h"
 #include "System/Exceptions.h"
 #include "System/ThreadPool.h"
-#if       !defined(DEDICATED) && !defined(UNITSYNC)
-#include "System/Platform/Watchdog.h"
-#endif // !defined(DEDICATED) && !defined(UNITSYNC)
+#include "System/Config/ConfigHandler.h"
+#include "System/FileSystem/RapidHandler.h"
+
+#if !defined(DEDICATED) && !defined(UNITSYNC)
+	#include "System/TimeProfiler.h"
+	#include "System/Platform/Watchdog.h"
+#endif
 
 
 #define LOG_SECTION_ARCHIVESCANNER "ArchiveScanner"
@@ -48,6 +52,7 @@ LOG_REGISTER_SECTION_GLOBAL(LOG_SECTION_ARCHIVESCANNER)
 const int INTERNAL_VER = 10;
 CArchiveScanner* archiveScanner = NULL;
 
+CONFIG(bool, FastArchiveScan).defaultValue(true).description("If enabled, only generate archive checksums on-demand.");
 
 
 /*
@@ -287,7 +292,7 @@ std::string CArchiveScanner::ArchiveData::GetInfoValueString(const std::string&
 		if (infoItem->valueType == INFO_VALUE_TYPE_STRING) {
 			valueString = infoItem->valueTypeString;
 		} else {
-			valueString = info_getValueAsString(infoItem);
+			valueString = infoItem->GetValueAsString();
 		}
 	}
 
@@ -350,17 +355,7 @@ CArchiveScanner::CArchiveScanner()
 		ReadCacheData(cacheFolder + "ArchiveCache.lua");
 	}
 
-	const std::vector<std::string>& datadirs = dataDirLocater.GetDataDirPaths();
-	std::vector<std::string> scanDirs;
-	for (auto d = datadirs.rbegin(); d != datadirs.rend(); ++d) {
-		scanDirs.push_back(*d + "maps");
-		scanDirs.push_back(*d + "base");
-		scanDirs.push_back(*d + "games");
-		scanDirs.push_back(*d + "packages");
-	}
-	// ArchiveCache has been parsed at this point --> archiveInfos is populated
-	ScanDirs(scanDirs, true);
-	WriteCacheData(GetFilepath());
+	ScanAllDirs();
 }
 
 
@@ -377,6 +372,24 @@ const std::string& CArchiveScanner::GetFilepath() const
 	return cachefile;
 }
 
+void CArchiveScanner::ScanAllDirs()
+{
+	const std::vector<std::string>& datadirs = dataDirLocater.GetDataDirPaths();
+	std::vector<std::string> scanDirs;
+	for (auto d = datadirs.rbegin(); d != datadirs.rend(); ++d) {
+		scanDirs.push_back(*d + "maps");
+		scanDirs.push_back(*d + "base");
+		scanDirs.push_back(*d + "games");
+		scanDirs.push_back(*d + "packages");
+	}
+	// ArchiveCache has been parsed at this point --> archiveInfos is populated
+#if !defined(DEDICATED) && !defined(UNITSYNC)
+	ScopedOnceTimer foo("CArchiveScanner");
+#endif
+	ScanDirs(scanDirs, !configHandler->GetBool("FastArchiveScan"));
+	WriteCacheData(GetFilepath());
+}
+
 
 void CArchiveScanner::ScanDirs(const std::vector<std::string>& scanDirs, bool doChecksum)
 {
@@ -530,60 +543,14 @@ static bool IsBaseContent(const std::string& fileName)
 
 void CArchiveScanner::ScanArchive(const std::string& fullName, bool doChecksum)
 {
+	unsigned modifiedTime;
+	if (CheckCachedData(fullName, &modifiedTime, doChecksum))
+		return;
+
 	const std::string fn    = FileSystem::GetFilename(fullName);
 	const std::string fpath = FileSystem::GetDirectory(fullName);
 	const std::string lcfn  = StringToLower(fn);
 
-	// Stat file
-	struct stat info = {0};
-	int statfailed = stat(fullName.c_str(), &info);
-
-	// If stat fails, assume the archive is not broken nor cached
-	if (!statfailed) {
-		// Determine whether this archive has earlier be found to be broken
-		std::map<std::string, BrokenArchive>::iterator bai = brokenArchives.find(lcfn);
-		if (bai != brokenArchives.end()) {
-			if ((unsigned)info.st_mtime == bai->second.modified && fpath == bai->second.path) {
-				bai->second.updated = true;
-				return;
-			}
-		}
-
-		// Determine whether to rely on the cached info or not
-		std::map<std::string, ArchiveInfo>::iterator aii = archiveInfos.find(lcfn);
-		if (aii != archiveInfos.end()) {
-			// This archive may have been obsoleted, do not process it if so
-			if (!aii->second.replaced.empty()) {
-				return;
-			}
-
-			if ((unsigned)info.st_mtime == aii->second.modified && fpath == aii->second.path) {
-				// cache found update checksum if wanted
-				aii->second.updated = true;
-				if (doChecksum && (aii->second.checksum == 0)) {
-					aii->second.checksum = GetCRC(fullName);
-				}
-				return;
-			} else {
-				if (aii->second.updated) {
-					const std::string filename = aii->first;
-					LOG_L(L_ERROR, "Found a \"%s\" already in \"%s\", ignoring.", fullName.c_str(), (aii->second.path + aii->second.origName).c_str());
-					if (IsBaseContent(filename)) {
-						throw user_error(std::string("duplicate base content detected:\n\t") + aii->second.path + std::string("\n\t") + fpath
-							+ std::string("\nPlease fix your configuration/installation as this can cause desyncs!"));
-					}
-					return;
-				}
-
-				// If we are here, we could have invalid info in the cache
-				// Force a reread if it is a directory archive (.sdd), as
-				// st_mtime only reflects changes to the directory itself,
-				// not the contents.
-				archiveInfos.erase(aii);
-			}
-		}
-	}
-
 	boost::scoped_ptr<IArchive> ar(archiveLoader.OpenArchive(fullName));
 	if (!ar || !ar->IsOpen()) {
 		LOG_L(L_WARNING, "Unable to open archive: %s", fullName.c_str());
@@ -591,7 +558,7 @@ void CArchiveScanner::ScanArchive(const std::string& fullName, bool doChecksum)
 		// record it as broken, so we don't need to look inside everytime
 		BrokenArchive& ba = brokenArchives[lcfn];
 		ba.path = fpath;
-		ba.modified = info.st_mtime;
+		ba.modified = modifiedTime;
 		ba.updated = true;
 		ba.problem = "Unable to open archive";
 		return;
@@ -626,7 +593,7 @@ void CArchiveScanner::ScanArchive(const std::string& fullName, bool doChecksum)
 		// record it as broken, so we don't need to look inside everytime
 		BrokenArchive& ba = brokenArchives[lcfn];
 		ba.path = fpath;
-		ba.modified = info.st_mtime;
+		ba.modified = modifiedTime;
 		ba.updated = true;
 		ba.problem = error;
 		return;
@@ -660,18 +627,82 @@ void CArchiveScanner::ScanArchive(const std::string& fullName, bool doChecksum)
 	}
 
 	ai.path = fpath;
-	ai.modified = info.st_mtime;
+	ai.modified = modifiedTime;
 	ai.origName = fn;
 	ai.updated = true;
 	ai.checksum = (doChecksum) ? GetCRC(fullName) : 0;
 	archiveInfos[lcfn] = ai;
 }
 
+
+bool CArchiveScanner::CheckCachedData(const std::string& fullName, unsigned* modified, bool doChecksum)
+{
+	const std::string fn    = FileSystem::GetFilename(fullName);
+	const std::string fpath = FileSystem::GetDirectory(fullName);
+	const std::string lcfn  = StringToLower(fn);
+
+	// If stat fails, assume the archive is not broken nor cached
+	struct stat info;
+	const int statfailed = stat(fullName.c_str(), &info);
+	*modified = info.st_mtime;
+	if (statfailed)
+		return false;
+
+	// Determine whether this archive has earlier be found to be broken
+	std::map<std::string, BrokenArchive>::iterator bai = brokenArchives.find(lcfn);
+	if (bai != brokenArchives.end()) {
+		if ((unsigned)info.st_mtime == bai->second.modified && fpath == bai->second.path) {
+			bai->second.updated = true;
+			return true;
+		}
+	}
+
+	// Determine whether to rely on the cached info or not
+	std::map<std::string, ArchiveInfo>::iterator aii = archiveInfos.find(lcfn);
+	if (aii != archiveInfos.end()) {
+		// This archive may have been obsoleted, do not process it if so
+		if (!aii->second.replaced.empty()) {
+			return true;
+		}
+
+		if ((unsigned)info.st_mtime == aii->second.modified && fpath == aii->second.path) {
+			// cache found update checksum if wanted
+			aii->second.updated = true;
+			if (doChecksum && (aii->second.checksum == 0)) {
+				aii->second.checksum = GetCRC(fullName);
+			}
+			return true;
+		}
+
+		if (aii->second.updated) {
+			const std::string filename = aii->first;
+			LOG_L(L_ERROR, "Found a \"%s\" already in \"%s\", ignoring.", fullName.c_str(), (aii->second.path + aii->second.origName).c_str());
+			if (IsBaseContent(filename)) {
+				throw user_error(std::string("duplicate base content detected:\n\t") + aii->second.path + std::string("\n\t") + fpath
+					+ std::string("\nPlease fix your configuration/installation as this can cause desyncs!"));
+			}
+			return true; // ignore
+		}
+
+		// If we are here, we could have invalid info in the cache
+		// Force a reread if it is a directory archive (.sdd), as
+		// st_mtime only reflects changes to the directory itself,
+		// not the contents.
+		archiveInfos.erase(aii);
+	}
+
+	return false;
+}
+
+
 bool CArchiveScanner::ScanArchiveLua(IArchive* ar, const std::string& fileName, ArchiveInfo& ai, std::string& err)
 {
 	std::vector<boost::uint8_t> buf;
 	if (!ar->GetFile(fileName, buf) || buf.empty()) {
-		err = "Error reading " + fileName + " from " + ar->GetArchiveName();
+		err = "Error reading " + fileName;
+		if (ar->GetArchiveName().find(".sdp") != std::string::npos) {
+			err += " (archive's rapid tag: " + GetRapidTagFromPackage(FileSystem::GetBasename(ar->GetArchiveName())) + ")";
+		}
 		return false;
 	}
 
@@ -794,6 +825,13 @@ unsigned int CArchiveScanner::GetCRC(const std::string& arcName)
 	return digest;
 }
 
+
+void CArchiveScanner::ComputeChecksumForArchive(const std::string& filePath)
+{
+	ScanArchive(filePath, true);
+}
+
+
 void CArchiveScanner::ReadCacheData(const std::string& filename)
 {
 	if (!FileSystem::FileExists(filename)) {
@@ -886,39 +924,19 @@ void CArchiveScanner::WriteCacheData(const std::string& filename)
 	}
 
 	// First delete all outdated information
-	// TODO: this pattern should be moved into an utility function..
-	for (std::map<std::string, ArchiveInfo>::iterator i = archiveInfos.begin(); i != archiveInfos.end(); ) {
-		if (!i->second.updated) {
-			i = set_erase(archiveInfos, i);
-		} else {
-			++i;
-		}
-	}
-	for (std::map<std::string, BrokenArchive>::iterator i = brokenArchives.begin(); i != brokenArchives.end(); ) {
-		if (!i->second.updated) {
-			i = set_erase(brokenArchives, i);
-		} else {
-			++i;
-		}
-	}
+	spring::map_erase_if(archiveInfos, [](const decltype(archiveInfos)::value_type& p) {
+		return !p.second.updated;
+	});
+	spring::map_erase_if(brokenArchives, [](const decltype(brokenArchives)::value_type& p) {
+		return !p.second.updated;
+	});
 
 	fprintf(out, "local archiveCache = {\n\n");
 	fprintf(out, "\tinternalver = %i,\n\n", INTERNAL_VER);
+	fprintf(out, "\tarchives = {  -- count = %u\n", unsigned(archiveInfos.size()));
 
-#ifdef _WIN64 //fprintf sometimes spews false warnings over %I64u
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wformat"
-#endif
-
-	fprintf(out, "\tarchives = {  -- count = " _STPF_ "\n", archiveInfos.size());
-
-#ifdef _WIN64
-#pragma GCC diagnostic pop
-#endif
-
-	std::map<std::string, ArchiveInfo>::const_iterator arcIt;
-	for (arcIt = archiveInfos.begin(); arcIt != archiveInfos.end(); ++arcIt) {
-		const ArchiveInfo& arcInfo = arcIt->second;
+	for (const auto& arcIt: archiveInfos) {
+		const ArchiveInfo& arcInfo = arcIt.second;
 
 		fprintf(out, "\t\t{\n");
 		SafeStr(out, "\t\t\tname = ",              arcInfo.origName);
@@ -932,22 +950,11 @@ void CArchiveScanner::WriteCacheData(const std::string& filename)
 		if (!archData.GetName().empty()) {
 			fprintf(out, "\t\t\tarchivedata = {\n");
 
-			const std::map<std::string, InfoItem>& info = archData.GetInfo();
-			std::map<std::string, InfoItem>::const_iterator ii;
-			for (ii = info.begin(); ii != info.end(); ++ii) {
-				switch (ii->second.valueType) {
-					case INFO_VALUE_TYPE_STRING: {
-						SafeStr(out, std::string("\t\t\t\t" + ii->first + " = ").c_str(), ii->second.valueTypeString);
-					} break;
-					case INFO_VALUE_TYPE_INTEGER: {
-						fprintf(out, "\t\t\t\t%s = %d,\n", ii->first.c_str(), ii->second.value.typeInteger);
-					} break;
-					case INFO_VALUE_TYPE_FLOAT: {
-						fprintf(out, "\t\t\t\t%s = %f,\n", ii->first.c_str(), ii->second.value.typeFloat);
-					} break;
-					case INFO_VALUE_TYPE_BOOL: {
-						fprintf(out, "\t\t\t\t%s = %d,\n", ii->first.c_str(), (int)ii->second.value.typeBool);
-					} break;
+			for (const auto& ii: archData.GetInfo()) {
+				if (ii.second.valueType == INFO_VALUE_TYPE_STRING) {
+					SafeStr(out, std::string("\t\t\t\t" + ii.first + " = ").c_str(), ii.second.valueTypeString);
+				} else {
+					fprintf(out, "\t\t\t\t%s = %s,\n", ii.first.c_str(), ii.second.GetValueAsString(false).c_str());
 				}
 			}
 
@@ -973,23 +980,13 @@ void CArchiveScanner::WriteCacheData(const std::string& filename)
 
 	fprintf(out, "\t},\n\n"); // close 'archives'
 
-#ifdef _WIN64 //fprintf sometimes spews false warnings over %I64u
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wformat"
-#endif
+	fprintf(out, "\tbrokenArchives = {  -- count = %u\n", unsigned(brokenArchives.size()));
 
-	fprintf(out, "\tbrokenArchives = {  -- count = " _STPF_ "\n", brokenArchives.size());
-
-#ifdef _WIN64
-#pragma GCC diagnostic pop
-#endif
-
-	std::map<std::string, BrokenArchive>::const_iterator bai;
-	for (bai = brokenArchives.begin(); bai != brokenArchives.end(); ++bai) {
-		const BrokenArchive& ba = bai->second;
+	for (const auto& bai: brokenArchives) {
+		const BrokenArchive& ba = bai.second;
 
 		fprintf(out, "\t\t{\n");
-		SafeStr(out, "\t\t\tname = ", bai->first);
+		SafeStr(out, "\t\t\tname = ", bai.first);
 		SafeStr(out, "\t\t\tpath = ", ba.path);
 		fprintf(out, "\t\t\tmodified = \"%u\",\n", ba.modified);
 		SafeStr(out, "\t\t\tproblem = ", ba.problem);
@@ -1008,13 +1005,11 @@ void CArchiveScanner::WriteCacheData(const std::string& filename)
 }
 
 
-static bool archNameCompare(const CArchiveScanner::ArchiveData& a, const CArchiveScanner::ArchiveData& b)
-{
-	return (a.GetNameVersioned() < b.GetNameVersioned());
-}
 static void sortByName(std::vector<CArchiveScanner::ArchiveData>& data)
 {
-	std::sort(data.begin(), data.end(), archNameCompare);
+	std::sort(data.begin(), data.end(), [](const CArchiveScanner::ArchiveData& a, const CArchiveScanner::ArchiveData& b){
+		return (a.GetNameVersioned() < b.GetNameVersioned());
+	});
 }
 
 std::vector<CArchiveScanner::ArchiveData> CArchiveScanner::GetPrimaryMods() const
@@ -1148,34 +1143,35 @@ std::string CArchiveScanner::MapNameToMapFile(const std::string& s) const
 	return s;
 }
 
-unsigned int CArchiveScanner::GetSingleArchiveChecksum(const std::string& name) const
+unsigned int CArchiveScanner::GetSingleArchiveChecksum(const std::string& filePath)
 {
-	std::string lcname = FileSystem::GetFilename(name);
+	ComputeChecksumForArchive(filePath);
+	std::string lcname = FileSystem::GetFilename(filePath);
 	StringToLowerInPlace(lcname);
 
 	std::map<std::string, ArchiveInfo>::const_iterator aii = archiveInfos.find(lcname);
 	if (aii == archiveInfos.end()) {
-		LOG_SL(LOG_SECTION_ARCHIVESCANNER, L_WARNING, "%s checksum: not found (0)", name.c_str());
+		LOG_SL(LOG_SECTION_ARCHIVESCANNER, L_WARNING, "%s checksum: not found (0)", filePath.c_str());
 		return 0;
 	}
 
-	LOG_S(LOG_SECTION_ARCHIVESCANNER,"%s checksum: %d/%u", name.c_str(), aii->second.checksum, aii->second.checksum);
+	LOG_S(LOG_SECTION_ARCHIVESCANNER,"%s checksum: %d/%u", filePath.c_str(), aii->second.checksum, aii->second.checksum);
 	return aii->second.checksum;
 }
 
-unsigned int CArchiveScanner::GetArchiveCompleteChecksum(const std::string& name) const
+unsigned int CArchiveScanner::GetArchiveCompleteChecksum(const std::string& name)
 {
 	const std::vector<std::string>& ars = GetAllArchivesUsedBy(name);
 	unsigned int checksum = 0;
 
-	for (unsigned int a = 0; a < ars.size(); a++) {
-		checksum ^= GetSingleArchiveChecksum(ars[a]);
+	for (const std::string& filePath: ars) {
+		checksum ^= GetSingleArchiveChecksum(filePath);
 	}
 	LOG_S(LOG_SECTION_ARCHIVESCANNER, "archive checksum %s: %d/%u", name.c_str(), checksum, checksum);
 	return checksum;
 }
 
-void CArchiveScanner::CheckArchive(const std::string& name, unsigned checksum) const
+void CArchiveScanner::CheckArchive(const std::string& name, unsigned checksum)
 {
 	unsigned localChecksum = GetArchiveCompleteChecksum(name);
 
