@@ -1,12 +1,14 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include <fstream>
+#include <sstream>
+#include <zlib.h>
 
 #include "ExternalAI/EngineOutHandler.h"
 #include "CregLoadSaveHandler.h"
 #include "Map/ReadMap.h"
 #include "Game/Game.h"
 #include "Game/GameSetup.h"
+#include "Game/GameVersion.h"
 #include "Game/GlobalUnsynced.h"
 #include "Game/WaitCommandsAI.h"
 #include "Net/GameServer.h"
@@ -21,18 +23,23 @@
 #include "Sim/Misc/Wind.h"
 #include "Sim/Projectiles/ProjectileHandler.h"
 #include "Sim/Units/CommandAI/BuilderCAI.h"
+#include "Sim/Units/CommandAI/CommandDescription.h"
+#include "Sim/Units/Scripts/CobEngine.h"
+#include "Sim/Units/Scripts/UnitScriptEngine.h"
+#include "Sim/Units/Scripts/NullUnitScript.h"
 #include "Game/UI/Groups/GroupHandler.h"
 
 #include "System/Platform/errorhandler.h"
 #include "System/FileSystem/DataDirsAccess.h"
 #include "System/FileSystem/FileQueryFlags.h"
+#include "System/FileSystem/GZFileHandler.h"
 #include "System/creg/Serializer.h"
 #include "System/Exceptions.h"
 #include "System/Log/ILog.h"
 
 
 CCregLoadSaveHandler::CCregLoadSaveHandler()
-	: ifs(NULL)
+	: iss(nullptr)
 {}
 
 CCregLoadSaveHandler::~CCregLoadSaveHandler()
@@ -63,6 +70,9 @@ void CGameStateCollector::Serialize(creg::ISerializer* s)
 	s->SerializeObjectInstance(readMap, readMap->GetClass());
 	s->SerializeObjectInstance(quadField, quadField->GetClass());
 	s->SerializeObjectInstance(unitHandler, unitHandler->GetClass());
+	s->SerializeObjectInstance(cobEngine, cobEngine->GetClass());
+	s->SerializeObjectInstance(unitScriptEngine, unitScriptEngine->GetClass());
+	s->SerializeObjectInstance(&CNullUnitScript::value, CNullUnitScript::value.GetClass());
 	s->SerializeObjectInstance(featureHandler, featureHandler->GetClass());
 	s->SerializeObjectInstance(losHandler, losHandler->GetClass());
 	s->SerializeObjectInstance(&interceptHandler, interceptHandler.GetClass());
@@ -75,6 +85,7 @@ void CGameStateCollector::Serialize(creg::ISerializer* s)
 	for (int a=0; a < teamHandler->ActiveTeams(); a++) {
 		s->SerializeObjectInstance(grouphandlers[a], grouphandlers[a]->GetClass());
 	}
+	s->SerializeObjectInstance(commandDescriptionCache, commandDescriptionCache->GetClass());
 	s->SerializeObjectInstance(eoh, eoh->GetClass());
 }
 
@@ -107,32 +118,41 @@ static void ReadString(std::istream& s, std::string& str)
 
 
 
-void CCregLoadSaveHandler::SaveGame(const std::string& file)
+void CCregLoadSaveHandler::SaveGame(const std::string& path)
 {
 #ifdef USING_CREG
 	LOG("Saving game");
 	try {
-		std::ofstream ofs(dataDirsAccess.LocateFile(file, FileQueryFlags::WRITE).c_str(), std::ios::out|std::ios::binary);
-		if (ofs.bad() || !ofs.is_open()) {
-			throw content_error("Unable to save game to file \"" + file + "\"");
-		}
+
+		std::stringstream oss;
 
 		// write our own header. SavePackage() will add its own
-		WriteString(ofs, gameSetup->setupText);
-		WriteString(ofs, modName);
-		WriteString(ofs, mapName);
+		WriteString(oss, SpringVersion::GetSync());
+		WriteString(oss, gameSetup->setupText);
+		WriteString(oss, modName);
+		WriteString(oss, mapName);
 
 		CGameStateCollector gsc = CGameStateCollector();
 
 		// save creg state
 		creg::COutputStreamSerializer os;
-		os.SavePackage(&ofs, &gsc, gsc.GetClass());
-		PrintSize("Game", ofs.tellp());
+		os.SavePackage(&oss, &gsc, gsc.GetClass());
+		PrintSize("Game", oss.tellp());
 
 		// save ai state
-		int aistart = ofs.tellp();
-		eoh->Save(&ofs);
-		PrintSize("AIs", ((int)ofs.tellp()) - aistart);
+		int aistart = oss.tellp();
+		eoh->Save(&oss);
+		PrintSize("AIs", ((int)oss.tellp()) - aistart);
+
+		gzFile file = gzopen(dataDirsAccess.LocateFile(path, FileQueryFlags::WRITE).c_str(), "wb9");
+		if (file == nullptr) {
+			LOG_L(L_ERROR, "Save failed: couldn't open file");
+			return;
+		}
+		const std::string data = oss.str();
+		gzwrite(file, data.c_str(), data.size());
+		gzflush(file, Z_FINISH);
+		gzclose(file);
 
 		//FIXME add lua state
 	} catch (const content_error& ex) {
@@ -152,9 +172,27 @@ void CCregLoadSaveHandler::SaveGame(const std::string& file)
 }
 
 /// this just loads the mapname and some other early stuff
-void CCregLoadSaveHandler::LoadGameStartInfo(const std::string& file)
+void CCregLoadSaveHandler::LoadGameStartInfo(const std::string& path)
 {
-	ifs = new std::ifstream(dataDirsAccess.LocateFile(FindSaveFile(file)), std::ios::in|std::ios::binary);
+	CGZFileHandler saveFile(dataDirsAccess.LocateFile(FindSaveFile(path)), SPRING_VFS_RAW_FIRST);
+	iss = new std::stringstream;
+	std::stringbuf *sbuf = iss->rdbuf();
+	char buf[4096];
+	int len;
+	while ((len = saveFile.Read(buf, sizeof(buf))) > 0)
+		sbuf->sputn(buf, len);
+
+	//Check for compatible save versions
+	std::string saveVersion;
+	ReadString(*iss, saveVersion);
+	if (saveVersion != SpringVersion::GetSync()) {
+		if (SpringVersion::IsRelease()) {
+			throw content_error("File was saved by an incompatible engine version: " + saveVersion);
+		} else {
+			LOG_L(L_WARNING, "File was saved by a different engine version: %s", saveVersion.c_str());
+		}
+	}
+
 
 	// in case these contained values alredy
 	// (this is the case when loading a game through the spring menu eg),
@@ -168,11 +206,11 @@ void CCregLoadSaveHandler::LoadGameStartInfo(const std::string& file)
 	mapName = "";
 
 	// read our own header.
-	ReadString(*ifs, scriptText);
-	ReadString(*ifs, modName);
-	ReadString(*ifs, mapName);
+	ReadString(*iss, scriptText);
+	ReadString(*iss, modName);
+	ReadString(*iss, mapName);
 
-	CGameSetup::LoadSavedScript(file, scriptText);
+	CGameSetup::LoadSavedScript(path, scriptText);
 }
 
 /// this should be called on frame 0 when the game has started
@@ -186,7 +224,7 @@ void CCregLoadSaveHandler::LoadGame()
 
 	// load creg state
 	creg::CInputStreamSerializer inputStream;
-	inputStream.LoadPackage(ifs, pGSC, gsccls);
+	inputStream.LoadPackage(iss, pGSC, gsccls);
 	assert(pGSC && gsccls == CGameStateCollector::StaticClass());
 
 	CGameStateCollector* gsc = static_cast<CGameStateCollector*>(pGSC);
@@ -194,7 +232,7 @@ void CCregLoadSaveHandler::LoadGame()
 	gsc = NULL;
 
 	// load ai state
-	eoh->Load(ifs);
+	eoh->Load(iss);
 	//for (int a=0; a < teamHandler->ActiveTeams(); a++) { // For old savegames
 	//	if (teamHandler->Team(a)->isDead && eoh->IsSkirmishAI(a)) {
 	//		eoh->DestroySkirmishAI(skirmishAIId(a), 2 /* = team died */);
@@ -202,8 +240,8 @@ void CCregLoadSaveHandler::LoadGame()
 	//}
 
 	// cleanup
-	delete ifs;
-	ifs = NULL;
+	delete iss;
+	iss = NULL;
 
 	gs->paused = false;
 	if (gameServer) {
