@@ -7,7 +7,8 @@
 CONFIG(bool, DisableDemoVersionCheck).defaultValue(false).description("Allow to play every replay file (may crash / cause undefined behaviour in replays)");
 #endif
 #include "System/Exceptions.h"
-#include "System/FileSystem/FileHandler.h"
+#include "System/FileSystem/GZFileHandler.h"
+#include "System/FileSystem/FileSystem.h"
 #include "System/Log/ILog.h"
 #include "System/Net/RawPacket.h"
 #include "Game/GameVersion.h"
@@ -21,7 +22,7 @@ CONFIG(bool, DisableDemoVersionCheck).defaultValue(false).description("Allow to
 CDemoReader::CDemoReader(const std::string& filename, float curTime)
 	: playbackDemo(NULL)
 {
-	playbackDemo = new CFileHandler(filename, SPRING_VFS_PWD_ALL);
+	playbackDemo = new CGZFileHandler(filename, SPRING_VFS_PWD_ALL);
 
 	if (!playbackDemo->FileExists()) {
 		// file not found -> exception
@@ -52,10 +53,9 @@ CDemoReader::CDemoReader(const std::string& filename, float curTime)
 	}
 
 	if (fileHeader.scriptSize != 0) {
-		char* buf = new char[fileHeader.scriptSize];
-		playbackDemo->Read(buf, fileHeader.scriptSize);
-		setupScript = std::string(buf, fileHeader.scriptSize);
-		delete[] buf;
+		std::vector<char> buf(fileHeader.scriptSize);
+		playbackDemo->Read(&buf[0], fileHeader.scriptSize);
+		setupScript = std::string(&buf[0], fileHeader.scriptSize);
 	}
 
 	playbackDemo->Read((char*)&chunkHeader, sizeof(chunkHeader));
