@@ -36,8 +36,9 @@ struct log_filter_section_compare {
 		return LOG_SECTION_COMPARE(section1, section2);
 	}
 };
-
-
+#ifdef __cplusplus
+}
+#endif
 
 namespace {
 	typedef std::set<const char*, log_filter_section_compare> secSet_t;
@@ -55,7 +56,7 @@ namespace {
 		return sections;
 	}
 
-	void log_filter_printSectionMinLevels(const char* func) {
+	void inline log_filter_printSectionMinLevels(const char* func) {
 		printf("[%s][caller=%s]\n", __FUNCTION__, func);
 
 		const auto& secLevels = log_filter_getSectionMinLevels();
@@ -66,6 +67,9 @@ namespace {
 	}
 }
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 
 static inline int log_filter_section_getDefaultMinLevel(const char* section) {
