@@ -468,6 +468,26 @@ namespace Threading {
 	#if defined(__USE_GNU) && !defined(WIN32)
 		//alternative: pthread_setname_np(pthread_self(), newname.c_str());
 		prctl(PR_SET_NAME, newname.c_str(), 0, 0, 0);
+	#elif _MSC_VER
+		const DWORD MS_VC_EXCEPTION = 0x406D1388;
+		#pragma pack(push,8)
+		struct THREADNAME_INFO
+		{
+			DWORD dwType; // Must be 0x1000.
+			LPCSTR szName; // Pointer to name (in user addr space).
+			DWORD dwThreadID; // Thread ID (-1=caller thread).
+			DWORD dwFlags; // Reserved for future use, must be zero.
+		} info;
+		#pragma pack(pop)
+		info.dwType = 0x1000;
+		info.szName = newname.c_str();
+		info.dwThreadID = (DWORD)-1;
+		info.dwFlags = 0;
+		__try {
+			RaiseException(MS_VC_EXCEPTION, 0, sizeof(info) / sizeof(ULONG_PTR), (ULONG_PTR*) &info);
+		}
+		__except (EXCEPTION_EXECUTE_HANDLER) {
+		}
 	#endif
 	}
 
