@@ -6,15 +6,8 @@
 #include <boost/cstdint.hpp> /* Replace with <stdint.h> if appropriate */
 
 #undef get16bits
-#if (defined(__GNUC__) && defined(__i386__)) || defined(__WATCOMC__) \
-  || defined(_MSC_VER) || defined (__BORLANDC__) || defined (__TURBOC__)
-#define get16bits(d) (*((const boost::uint16_t *) (d)))
-#endif
+#define get16bits(d) (*reinterpret_cast<const boost::uint16_t*>(d))
 
-#if !defined (get16bits)
-#define get16bits(d) ((((boost::uint32_t)(((const boost::uint8_t *)(d))[1])) << 8)\
-	+(boost::uint32_t)(((const boost::uint8_t *)(d))[0]) )
-#endif
 
 /** @brief a fast hash function
  *
@@ -23,21 +16,19 @@
  */
 static inline boost::uint32_t HsiehHash (const void* data_, int len, boost::uint32_t hash)
 {
-	const char* data = static_cast<const char*>(data_);
+	const boost::uint8_t* data = reinterpret_cast<const boost::uint8_t*>(data_);
+	assert(data != nullptr || len == 0);
 
-	if (len <= 0 || data == NULL) return hash;
-
-	boost::uint32_t tmp;
 	int rem = len & 3;
 	len >>= 2;
 
 	/* Main loop */
 	for (;len > 0; len--) {
 		hash  += get16bits (data);
-		tmp    = (get16bits (data+2) << 11) ^ hash;
+		boost::uint32_t tmp = (get16bits (data+2) << 11) ^ hash;
 		hash   = (hash << 16) ^ tmp;
-		data  += 2*sizeof (boost::uint16_t);
 		hash  += hash >> 11;
+		data  += 2*sizeof (boost::uint16_t);
 	}
 
 	/* Handle end cases */
@@ -58,6 +49,8 @@ static inline boost::uint32_t HsiehHash (const void* data_, int len, boost::uint
 		hash ^= hash << 10;
 		hash += hash >> 1;
 		break;
+	default:
+		break;
 	}
 
 	/* Force "avalanching" of final 127 bits */
