@@ -28,6 +28,7 @@ namespace creg {
 					elemType->Serialize(s, &*it);
 				}
 			} else {
+				ct.clear();
 				int size;
 				s->SerializeInt(&size, sizeof(int));
 				ct.resize(size);
@@ -48,8 +49,7 @@ namespace creg {
 	template<typename T>
 	struct DeduceType< std::list<T> > {
 		static boost::shared_ptr<IType> Get() {
-			DeduceType<T> elemtype;
-			return boost::shared_ptr<IType>(new ListType< std::list<T> >(elemtype.Get()));
+			return boost::shared_ptr<IType>(new ListType< std::list<T> >(DeduceType<T>::Get()));
 		}
 	};
 }
