@@ -9,6 +9,7 @@
 #include <string>
 #include <type_traits> // std::is_polymorphic
 #include <boost/shared_ptr.hpp>
+#include <functional>
 
 #include "ISerializer.h"
 #include "System/Sync/SyncedPrimitive.h"
@@ -22,6 +23,18 @@ namespace creg {
 
 	typedef unsigned int uint;
 
+	enum ClassFlags {
+		CF_None = 0,
+		CF_Abstract = 4,
+		CF_Synced = 8,
+	};
+
+	/** Class member flags to use with CR_MEMBER_SETFLAG */
+	enum ClassMemberFlag {
+		CM_NoSerialize = 1, /// Make the serializers skip the member
+		CM_Config = 2,  // Exposed in config
+	};
+
 	// Fundamental/basic types
 	enum BasicTypeID
 	{
@@ -37,7 +50,7 @@ namespace creg {
 	{
 	public:
 		// Type interface can go here...
-		virtual ~IType();
+		virtual ~IType() {}
 
 		virtual void Serialize(ISerializer* s, void* instance) = 0;
 		virtual std::string GetName() const = 0;
@@ -46,82 +59,10 @@ namespace creg {
 		static boost::shared_ptr<IType> CreateBasicType(BasicTypeID t, size_t size);
 		static boost::shared_ptr<IType> CreateStringType();
 		static boost::shared_ptr<IType> CreateObjInstanceType(Class* objectType);
+		static boost::shared_ptr<IType> CreateIgnoredType(size_t size);
 	};
 
-	class IMemberRegistrator
-	{
-	public:
-		virtual ~IMemberRegistrator();
-		virtual void RegisterMembers(Class* cls) = 0;
-	};
 
-	enum ClassFlags {
-		CF_None = 0,
-		CF_Abstract = 4,
-		CF_Synced = 8,
-	};
-
-	/** Class member flags to use with CR_MEMBER_SETFLAG */
-	enum ClassMemberFlag {
-		CM_NoSerialize = 1, /// Make the serializers skip the member
-		CM_Config = 2,  // Exposed in config
-	};
-
-/**
- * Stores class bindings such as constructor/destructor
- */
-	class ClassBinder
-	{
-	public:
-		ClassBinder(const char* className, unsigned int cf, ClassBinder* base,
-				IMemberRegistrator** mreg, int instanceSize, int instanceAlignment, bool hasVTable, bool isCregStruct,
-				void (*constructorProc)(void* instance),
-				void (*destructorProc)(void* instance));
-
-		Class* class_;
-		ClassBinder* base;
-		ClassFlags flags;
-		IMemberRegistrator** memberRegistrator;
-		const char* name;
-		int size; // size of an instance in bytes
-		int alignment;
-		bool hasVTable;
-		bool isCregStruct;
-
-		void (*constructor)(void* instance);
-		/**
-		 * Needed for classes without virtual destructor.
-		 * (classes/structs declared with CR_DECLARE_STRUCT)
-		 */
-		void (*destructor)(void* instance);
-
-		ClassBinder* nextBinder;
-	};
-
-	class System
-	{
-	public:
-		/// Return the global list of classes
-		static const std::vector<Class*>& GetClasses() { return classes; }
-
-		/**
-		 * Initialization of creg, collects all the classes and initializes
-		 * metadata.
-		 */
-		static void InitializeClasses();
-
-		/// Shutdown of creg
-		static void FreeClasses();
-
-		/// Find a class by name
-		static Class* GetClass(const std::string& name);
-
-		static void AddClassBinder(ClassBinder* cb);
-
-	protected:
-		static ClassBinder* binderList;
-		static std::vector<Class*> classes;
-	};
 
 	/**
 	 * Represents a C++ class or struct, declared with
@@ -139,206 +80,125 @@ namespace creg {
 			int flags; // combination of ClassMemberFlag's
 		};
 
-		Class();
-		virtual ~Class();
+		Class(const char* name);
 
 		/// Returns true if this class is equal to or derived from other
 		bool IsSubclassOf(Class* other) const;
-		void DeleteInstance(void* inst);
+
 		/// Allocate an instance of the class
 		void* CreateInstance();
+		void DeleteInstance(void* inst);
+
 		/// Calculate a checksum from the class metadata
 		void CalculateChecksum(unsigned int& checksum);
-		bool AddMember(const char* name, boost::shared_ptr<IType> type, unsigned int offset, int alignment);
-		bool AddMember(const char* name, IType* type, unsigned int offset, int alignment);
+		void AddMember(const char* name, boost::shared_ptr<IType> type, unsigned int offset, int alignment);
 		void SetMemberFlag(const char* name, ClassMemberFlag f);
-		Member* FindMember(const char* name);
+		Member* FindMember(const char* name, const bool inherited = true);
 
 		void BeginFlag(ClassMemberFlag flag);
 		void EndFlag(ClassMemberFlag flag);
-
 		void SetFlag(ClassFlags flag);
 
-		bool IsAbstract() const { return (binder->flags & CF_Abstract) != 0; }
-
-		virtual void CallSerializeProc(void* object, ISerializer* s) = 0;
-		virtual void CallPostLoadProc(void* object) = 0;
-		virtual bool HasSerialize() = 0;
-		virtual bool HasPostLoad() = 0;
+		inline bool IsAbstract() const;
+		inline Class* base() const;
 
 		/// Returns all concrete classes that implement this class
 		std::vector<Class*> GetImplementations();
 
-		std::vector <Member*> members;
 		/// all classes that derive from this class
-		std::vector <Class*> derivedClasses;
+		const std::vector<Class*>& GetDerivedClasses() const;
+
+		template<class T, typename TF>
+		void SetSerialize(T* foo, TF proc) {
+			serializeProc = [=](void* object, ISerializer* s) {
+				(static_cast<T*>(object)->*proc)(s);
+			};
+		}
+		template<class T, typename TF>
+		void SetPostLoad(T* foo, TF proc) {
+			postLoadProc = [=](void* object) {
+				(static_cast<T*>(object)->*proc)();
+			};
+		}
+		bool HasSerialize() const { return (bool)serializeProc; }
+		bool HasPostLoad() const { return (bool)postLoadProc; }
+		void CallSerializeProc(void* object, ISerializer* s) { serializeProc(object, s); }
+		void CallPostLoadProc(void* object)                  { postLoadProc(object); }
+
+	public:
+		std::vector<Member> members;
+
 		ClassBinder* binder;
 		std::string name;
 		int size;
 		int alignment;
-		Class* base;
 
-		friend class ClassBinder;
+		std::function<void(void* object, ISerializer* s)> serializeProc;
+		std::function<void(void* object)> postLoadProc;
+
+	private:
+		int currentMemberFlags;
 	};
 
-	template<class T>
-	class ClassStrong : public Class
+
+	/**
+	 * Stores class bindings such as constructor/destructor
+	 */
+	class ClassBinder //TODO merge with `class Class`
 	{
 	public:
-		ClassStrong() :
-			serializeProc(NULL),
-			postLoadProc(NULL) { }
+		ClassBinder(const char* className, ClassFlags cf, ClassBinder* base,
+				void (*memberRegistrator)(creg::Class*),
+				int instanceSize, int instanceAlignment, bool hasVTable, bool isCregStruct,
+				void (*constructorProc)(void* instance), void (*destructorProc)(void* instance));
 
-		virtual ~ClassStrong() { }
+		Class class_;
+		ClassBinder* base;
+		ClassFlags flags;
+		const char* name;
+		int size; // size of an instance in bytes
+		int alignment;
+		bool hasVTable;
+		bool isCregStruct;
 
-		virtual void CallSerializeProc(void* object, ISerializer* s)
-		{
-			(static_cast<T*>(object)->*serializeProc)(s);
-		}
-
-		virtual void CallPostLoadProc(void* object)
-		{
-			(static_cast<T*>(object)->*postLoadProc)();
-		}
-
-		virtual bool HasSerialize()
-		{
-			return !!serializeProc;
-		}
-
-		virtual bool HasPostLoad()
-		{
-			return !!postLoadProc;
-		}
-
-		void (T::*serializeProc)(ISerializer* s);
-		void (T::*postLoadProc)();
+		void (*constructor)(void* instance);
+		/**
+		 * Needed for classes without virtual destructor.
+		 * (classes/structs declared with CR_DECLARE_STRUCT)
+		 */
+		void (*destructor)(void* instance);
 	};
 
+
+	/**
+	 * CREG main interface for outside code
+	 */
+	class System
+	{
+	public:
+		/// Return the global list of classes
+		static const std::vector<Class*>& GetClasses();
+
+		/// Find a class by name
+		static Class* GetClass(const std::string& name);
+
+		static void AddClass(Class* c);
+	};
+
+
+	//Note: has to be defined after `class ClassBinder`, else we get a compile error
+	bool Class::IsAbstract() const { return (binder->flags & CF_Abstract) != 0; }
+	Class* Class::base() const { return binder->base ? &binder->base->class_ : nullptr; }
+}
+
+
+
+
 // -------------------------------------------------------------------
 // Container Type templates
 // -------------------------------------------------------------------
 
-	// vector,deque container
-	template<typename T>
-	class DynamicArrayType : public IType
-	{
-	public:
-		typedef typename T::iterator iterator;
-		typedef typename T::value_type ElemT;
-
-		boost::shared_ptr<IType> elemType;
-
-		DynamicArrayType(boost::shared_ptr<IType> et)
-			: elemType(et) {}
-		~DynamicArrayType() {}
-
-		void Serialize(ISerializer* s, void* inst) {
-			T& ct = *(T*)inst;
-			if (s->IsWriting()) {
-				int size = (int)ct.size();
-				s->SerializeInt(&size, sizeof(int));
-				for (int a = 0; a < size; a++) {
-					elemType->Serialize(s, &ct[a]);
-				}
-			} else {
-				int size;
-				s->SerializeInt(&size, sizeof(int));
-				ct.resize(size);
-				for (int a = 0; a < size; a++) {
-					elemType->Serialize(s, &ct[a]);
-				}
-			}
-		}
-		std::string GetName() const { return elemType->GetName() + "[]"; }
-		size_t GetSize() const { return sizeof(T); }
-	};
-
-	class StaticArrayBaseType : public IType
-	{
-	public:
-		boost::shared_ptr<IType> elemType;
-		int size, elemSize;
-
-		StaticArrayBaseType(boost::shared_ptr<IType> et, int Size, int ElemSize)
-			: elemType(et), size(Size), elemSize(ElemSize) {}
-		~StaticArrayBaseType() {}
-
-		std::string GetName() const;
-		size_t GetSize() const { return size * elemSize; }
-	};
-
-	template<typename T, int Size>
-	class StaticArrayType : public StaticArrayBaseType
-	{
-	public:
-		typedef T ArrayType[Size];
-		StaticArrayType(boost::shared_ptr<IType> et)
-			: StaticArrayBaseType(et, Size, sizeof(ArrayType)/Size) {}
-		void Serialize(ISerializer* s, void* instance)
-		{
-			T* array = (T*)instance;
-			for (int a = 0; a < Size; a++)
-				elemType->Serialize(s, &array[a]);
-		}
-	};
-
-	template<typename T>
-	class BitArrayType : public IType
-	{
-	public:
-		boost::shared_ptr<IType> elemType;
-
-		BitArrayType(boost::shared_ptr<IType> et)
-			: elemType(et) {}
-		~BitArrayType() {}
-
-		void Serialize(ISerializer* s, void* inst) {
-			T* ct = (T*)inst;
-			if (s->IsWriting()) {
-				int size = (int)ct->size();
-				s->SerializeInt(&size, sizeof(int));
-				for (int a = 0; a < size; a++) {
-					bool b = (*ct)[a];
-					elemType->Serialize(s, &b);
-				}
-			} else {
-				int size;
-				s->SerializeInt(&size, sizeof(int));
-				ct->resize(size);
-				for (int a = 0; a < size; a++) {
-					bool b;
-					elemType->Serialize(s, &b);
-					(*ct)[a] = b;
-				}
-			}
-		}
-		std::string GetName() const { return elemType->GetName() + "[]"; }
-		size_t GetSize() const { return sizeof(T); }
-	};
-
-	class IgnoredType : public IType
-	{
-	public:
-		int size;
-		IgnoredType(int Size) {size=Size;}
-		~IgnoredType() {}
-
-		void Serialize(ISerializer* s, void* instance)
-		{
-			for (int a=0;a<size;a++) {
-				char c=0;
-				s->Serialize(&c,1);
-			}
-		}
-		std::string GetName() const
-		{
-			return "ignored";
-		}
-		size_t GetSize() const { return size; }
-	};
-}
-
+#include "BasicTypes.h"
 #include "TypeDeduction.h"
 
 
@@ -346,20 +206,17 @@ namespace creg {
 #define offsetof_creg(type, member) (std::size_t)(((char*)&null->member) - ((char*)null))
 
 
-namespace creg {
-
 #define CR_DECLARE_BASE(TCls, isStr, VIRTUAL, OVERRIDE)	\
-public:					\
-	static creg::ClassBinder binder;				\
-	typedef TCls MyType;							\
-	static creg::IMemberRegistrator* memberRegistrator;	 \
-	static void _ConstructInstance(void* d);			\
-	static void _DestructInstance(void* d);			\
-	friend struct TCls##MemberRegistrator;			\
-	inline static creg::Class* StaticClass() { return binder.class_; } \
-	VIRTUAL creg::Class* GetClass() const OVERRIDE; \
+public: \
+	static creg::ClassBinder binder; \
 	static bool creg_hasVTable; \
-	static const bool creg_isStruct = isStr;
+	static const bool creg_isStruct = isStr; \
+	typedef TCls MyType; \
+	static creg::Class* StaticClass() { return &binder.class_; } \
+	VIRTUAL creg::Class* GetClass() const OVERRIDE { return &binder.class_; } \
+	static void _ConstructInstance(void* d); \
+	static void _DestructInstance(void* d); \
+	static void _CregRegisterMembers(creg::Class* class_);
 
 
 /** @def CR_DECLARE
@@ -369,7 +226,7 @@ public:					\
 #define CR_DECLARE(TCls) CR_DECLARE_BASE(TCls, false, virtual, )
 
 
-/** @def CR_DECLARE_STRUCT
+/** @def CR_DECLARE_DERIVED
  * Use this to declare a derived class
  * this should be put in the class definition, instead of CR_DECLARE
  */
@@ -390,6 +247,18 @@ public:					\
 #define CR_DECLARE_SUB(cl) \
 	struct cl##MemberRegistrator;
 
+/** @def CR_BIND
+ * Bind a class not derived from CObject
+ * should be used in the source file
+ * @param TCls class to bind
+ * @param ctor_args constructor arguments
+ */
+#define CR_BIND(TCls, ctor_args) \
+	bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
+	void TCls::_ConstructInstance(void* d) { new(d) MyType ctor_args; } \
+	void TCls::_DestructInstance(void* d) { ((MyType*)d)->~MyType(); } \
+	creg::ClassBinder TCls::binder(#TCls, creg::CF_None, nullptr, &TCls::_CregRegisterMembers, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance);
+
 /** @def CR_BIND_DERIVED
  * Bind a derived class declared with CR_DECLARE to creg
  * Should be used in the source file
@@ -399,11 +268,9 @@ public:					\
  */
 #define CR_BIND_DERIVED(TCls, TBase, ctor_args) \
 	bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
-	creg::IMemberRegistrator* TCls::memberRegistrator=0;	\
-	creg::Class* TCls::GetClass() const { return binder.class_; } \
 	void TCls::_ConstructInstance(void* d) { new(d) MyType ctor_args; } \
 	void TCls::_DestructInstance(void* d) { ((MyType*)d)->~MyType(); } \
-	creg::ClassBinder TCls::binder(#TCls, 0, &TBase::binder, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance);
+	creg::ClassBinder TCls::binder(#TCls, creg::CF_None, &TBase::binder, &TCls::_CregRegisterMembers, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance);
 
 /** @def CR_BIND_DERIVED_SUB
  * Bind a derived class inside another class to creg
@@ -415,44 +282,21 @@ public:					\
  */
 #define CR_BIND_DERIVED_SUB(TSuper, TCls, TBase, ctor_args) \
 	bool TSuper::TCls::creg_hasVTable = std::is_polymorphic<TSuper::TCls>::value; \
-	creg::IMemberRegistrator* TSuper::TCls::memberRegistrator=0;	 \
-	creg::Class* TSuper::TCls::GetClass() const { return binder.class_; }  \
 	void TSuper::TCls::_ConstructInstance(void* d) { new(d) TCls ctor_args; }  \
 	void TSuper::TCls::_DestructInstance(void* d) { ((TCls*)d)->~TCls(); }  \
-	creg::ClassBinder TSuper::TCls::binder(#TSuper "::" #TCls, 0, &TBase::binder, &TSuper::TCls::memberRegistrator, sizeof(TSuper::TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TSuper::TCls::_ConstructInstance, TSuper::TCls::_DestructInstance);
+	creg::ClassBinder TSuper::TCls::binder(#TSuper "::" #TCls, creg::CF_None, &TBase::binder, &TSuper::TCls::_CregRegisterMembers, sizeof(TSuper::TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TSuper::TCls::_ConstructInstance, TSuper::TCls::_DestructInstance);
 
-/** @def CR_BIND
- * Bind a class not derived from CObject
- * should be used in the source file
- * @param TCls class to bind
- * @param ctor_args constructor arguments
+
+/** @def CR_BIND_TEMPLATE
+ *  @see CR_BIND
  */
-#define CR_BIND(TCls, ctor_args) \
-	bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
-	creg::IMemberRegistrator* TCls::memberRegistrator=0;	\
-	creg::Class* TCls::GetClass() const { return binder.class_; } \
-	void TCls::_ConstructInstance(void* d) { new(d) MyType ctor_args; } \
-	void TCls::_DestructInstance(void* d) { ((MyType*)d)->~MyType(); } \
-	creg::ClassBinder TCls::binder(#TCls, 0, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance);
+#define CR_BIND_TEMPLATE(TCls, ctor_args) \
+	template<> bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
+	template<> void TCls::_ConstructInstance(void* d) { new(d) MyType ctor_args; } \
+	template<> void TCls::_DestructInstance(void* d) { ((MyType*)d)->~MyType(); } \
+	template<> void TCls::_CregRegisterMembers(creg::Class* class_); \
+	template<> creg::ClassBinder TCls::binder(#TCls, creg::CF_None, nullptr, &TCls::_CregRegisterMembers, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance);
 
-#ifdef __clang__
-	// LLVM/Clang expects a different order
-	#define CR_BIND_TEMPLATE(TCls, ctor_args) \
-		template<> bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
-		template<> creg::IMemberRegistrator* TCls::memberRegistrator=0; \
-		template<> void TCls::_ConstructInstance(void* d) { new(d) MyType ctor_args; } \
-		template<> void TCls::_DestructInstance(void* d) { ((MyType*)d)->~MyType(); } \
-		template<> creg::ClassBinder TCls::binder(#TCls, 0, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance); \
-		template<> creg::Class* TCls::GetClass() const { return binder.class_; }
-#else
-	#define CR_BIND_TEMPLATE(TCls, ctor_args) \
-		template<> bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
-		template<> creg::IMemberRegistrator* TCls::memberRegistrator=0; \
-		template<> creg::Class* TCls::GetClass() const { return binder.class_; } \
-		template<> void TCls::_ConstructInstance(void* d) { new(d) MyType ctor_args; } \
-		template<> void TCls::_DestructInstance(void* d) { ((MyType*)d)->~MyType(); } \
-		template<> creg::ClassBinder TCls::binder(#TCls, 0, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance);
-#endif
 
 /** @def CR_BIND_DERIVED_INTERFACE
  * Bind an abstract derived class
@@ -462,9 +306,7 @@ public:					\
  */
 #define CR_BIND_DERIVED_INTERFACE(TCls, TBase)	\
 	bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
-	creg::IMemberRegistrator* TCls::memberRegistrator=0;	\
-	creg::Class* TCls::GetClass() const { return binder.class_; } \
-	creg::ClassBinder TCls::binder(#TCls, (unsigned int)creg::CF_Abstract, &TBase::binder, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, 0, 0);
+	creg::ClassBinder TCls::binder(#TCls, creg::CF_Abstract, &TBase::binder, &TCls::_CregRegisterMembers, sizeof(TCls&), alignof(TCls&), TCls::creg_hasVTable, TCls::creg_isStruct, 0, 0);
 
 /** @def CR_BIND_INTERFACE
  * Bind an abstract class
@@ -475,9 +317,7 @@ public:					\
  */
 #define CR_BIND_INTERFACE(TCls)	\
 	bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
-	creg::IMemberRegistrator* TCls::memberRegistrator=0;	\
-	creg::Class* TCls::GetClass() const { return binder.class_; } \
-	creg::ClassBinder TCls::binder(#TCls, (unsigned int)creg::CF_Abstract, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, 0, 0);
+	creg::ClassBinder TCls::binder(#TCls, creg::CF_Abstract, nullptr, &TCls::_CregRegisterMembers, sizeof(TCls&), alignof(TCls&), TCls::creg_hasVTable, TCls::creg_isStruct, 0, 0);
 
 /** @def CR_REG_METADATA
  * Binds the class metadata to the class itself
@@ -491,37 +331,37 @@ public:					\
  * @see CR_POSTLOAD
  * @see CR_MEMBER_SETFLAG
  */
-#define CR_REG_METADATA(TClass, Members)				\
-	struct TClass##MemberRegistrator : creg::IMemberRegistrator {\
-	typedef TClass Type;						\
-	TClass##MemberRegistrator() {				\
-		Type::memberRegistrator=this;				\
-	}												\
-	void RegisterMembers(creg::Class* class_base_) {		\
-		creg::ClassStrong<TClass>* class_ = static_cast<creg::ClassStrong<TClass>*>(class_base_); \
-		Type* null=nullptr;						\
-		(void)null; /*suppress compiler warning if this isn't used*/	\
-		(void)class_; /*suppress compiler warning if this isn't used*/	\
-		Members; }									\
-	} static TClass##mreg;
+#define CR_REG_METADATA(TClass, Members) \
+	void TClass::_CregRegisterMembers(creg::Class* class_) { \
+		typedef TClass Type; \
+		Type* null=nullptr; \
+		(void)null; /*suppress compiler warning if this isn't used*/ \
+		Members; \
+	}
+
+/** @def CR_REG_METADATA_TEMPLATE
+ * Just like CR_REG_METADATA, but for a template.
+ *  @see CR_REG_METADATA
+ */
+#define CR_REG_METADATA_TEMPLATE(TCls, Members) \
+	template<> void TCls::_CregRegisterMembers(creg::Class* class_) { \
+		typedef TCls Type; \
+		Type* null=nullptr; \
+		(void)null; /*suppress compiler warning if this isn't used*/ \
+		Members; \
+	}
 
 /** @def CR_REG_METADATA_SUB
  * Just like CR_REG_METADATA, but for a subclass.
  *  @see CR_REG_METADATA
  */
-#define CR_REG_METADATA_SUB(TSuperClass, TSubClass, Members)				\
-	struct TSuperClass::TSubClass##MemberRegistrator : creg::IMemberRegistrator {\
-	typedef TSuperClass::TSubClass Type;						\
-	TSubClass##MemberRegistrator() {				\
-		Type::memberRegistrator=this;				\
-	}												\
-	void RegisterMembers(creg::Class* class_base_) { \
-		creg::ClassStrong<TSubClass>* class_ = static_cast<creg::ClassStrong<TSubClass>*>(class_base_); \
+#define CR_REG_METADATA_SUB(TSuperClass, TSubClass, Members) \
+	void TSuperClass::TSubClass::_CregRegisterMembers(creg::Class* class_) { \
+		typedef TSuperClass::TSubClass Type; \
 		Type* null=nullptr; \
-		(void)null; /*suppress compiler warning if this isn't used*/	\
-		(void)class_; /*suppress compiler warning if this isn't used*/	\
-		Members; } \
-	} static TSuperClass##TSubClass##mreg;
+		(void)null; /*suppress compiler warning if this isn't used*/ \
+		Members; \
+	}
 
 /** @def CR_MEMBER
  * Registers a class/struct member variable, of a type that is:
@@ -546,7 +386,7 @@ public:					\
  * Registers a member variable that isn't saved/loaded
  */
 #define CR_IGNORED(Member) \
-	class_->AddMember( #Member, new creg::IgnoredType(sizeof(Type::Member)), offsetof_creg(Type, Member), alignof(decltype(Type::Member)))
+	class_->AddMember( #Member, creg::IType::CreateIgnoredType(sizeof(Type::Member)), offsetof_creg(Type, Member), alignof(decltype(Type::Member)))
 
 
 /** @def CR_MEMBER_UN
@@ -555,7 +395,7 @@ public:					\
  * Currently works as CR_IGNORED.
  */
 #define CR_MEMBER_UN(Member) \
-    CR_IGNORED( Member )
+	CR_IGNORED( Member )
 
 /** @def CR_SETFLAG
  * Set a flag for a class/struct.
@@ -591,7 +431,7 @@ public:					\
  *   class
  */
 #define CR_SERIALIZER(SerializeFunc) \
-	(class_->serializeProc = &Type::SerializeFunc)
+	(class_->SetSerialize(null, &Type::SerializeFunc))
 
 /** @def CR_POSTLOAD
  * Registers a custom post-loading method for the class/struct
@@ -600,7 +440,6 @@ public:					\
  * There can only be one postload method per class/struct
  */
 #define CR_POSTLOAD(PostLoadFunc) \
-	(class_->postLoadProc = &Type::PostLoadFunc)
-}
+	(class_->SetPostLoad(null, &Type::PostLoadFunc))
 
 #endif // _CREG_H
