@@ -29,7 +29,7 @@ void Picture::Load(const std::string& _file)
 	
 	CBitmap bmp;
 	if (bmp.Load(file)) {
-		texture = bmp.CreateTexture(false);
+		texture = bmp.CreateTexture();
 	} else {
 		LOG_L(L_WARNING, "Failed to load: %s", file.c_str());
 		texture = 0;
