@@ -5,12 +5,12 @@
 
 #undef GL_GLEXT_LEGACY
 #define GL_GLEXT_PROTOTYPES
-#define _GDI32_
-
-#ifdef _MSC_VER
-// http://stackoverflow.com/questions/24186250/include-windows-h-causes-a-lot-of-syntax-errors
-#include <windef.h>
-#include <wingdi.h>
+#ifdef _WIN32
+# define _GDI32_
+# ifdef _DLL
+#  undef _DLL
+# endif
+# include <windows.h>
 #endif
 
 #include <GL/gl.h>
