@@ -36,23 +36,48 @@ CHttpDownloader::~CHttpDownloader()
 {
 }
 
-static size_t
-WriteMemoryCallback(void *contents, size_t size, size_t nmemb, void *userp)
+static size_t WriteMemoryCallback(void *contents, size_t size, size_t nmemb, void *userp)
 {
-	size_t realsize = size*nmemb;
+	const size_t realsize = size * nmemb;
 	std::string* res = static_cast<std::string*>(userp);
 	res->append((char*)contents, realsize);
 	return realsize;
 }
 
+static int progress_func(DownloadData* data, double total, double done, double, double)
+{
+	data->download->progress = done;
+	if (IDownloader::listener != NULL) {
+		IDownloader::listener(done, total);
+	}
+	if (data->got_ranges) {
+		LOG_PROGRESS(done, total, done >= total);
+	}
+	return 0;
+}
+
+
 //downloads url into res
 bool CHttpDownloader::DownloadUrl(const std::string& url, std::string& res)
 {
 	CurlWrapper* curlw = new CurlWrapper();
+	DownloadData d;
+	d.got_ranges = false;
+	d.download = new IDownload();
+	d.download->addMirror(url);
+	d.download->name = url;
+	d.download->origin_name = url;
+
 	curl_easy_setopt(curlw->GetHandle(), CURLOPT_URL, CurlWrapper::escapeUrl(url).c_str());
 	curl_easy_setopt(curlw->GetHandle(), CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
 	curl_easy_setopt(curlw->GetHandle(), CURLOPT_WRITEDATA, (void *)&res);
+	curl_easy_setopt(curlw->GetHandle(), CURLOPT_PROGRESSDATA, &d);
+	curl_easy_setopt(curlw->GetHandle(), CURLOPT_PROGRESSFUNCTION, progress_func);
+	curl_easy_setopt(curlw->GetHandle(), CURLOPT_NOPROGRESS, 0L);
 	CURLcode curlres = curl_easy_perform(curlw->GetHandle());
+
+	delete d.download;
+	d.download = nullptr;
 	if (curlres != CURLE_OK) {
 	        LOG_ERROR("Error in curl %s", curl_easy_strerror(curlres));
 	}
@@ -120,7 +145,7 @@ bool CHttpDownloader::ParseResult(const std::string& name, const std::string& js
 			LOG_ERROR("Invalid type in result");
 			return false;
 		}
-		filename.append(resfile["filename"].asString());
+		filename.append(CFileSystem::EscapeFilename(resfile["filename"].asString()));
 
 		const DownloadEnum::Category cat = DownloadEnum::getCatFromStr(category);
 		IDownload* dl=new IDownload(filename, springname, cat);
@@ -158,6 +183,7 @@ bool CHttpDownloader::ParseResult(const std::string& name, const std::string& js
 		}
 		res.push_back(dl);
 	}
+	LOG_DEBUG("Parsed %d results", res.size());
 	return true;
 }
 
@@ -173,7 +199,7 @@ bool CHttpDownloader::search(std::list<IDownload*>& res, const std::string& name
 	return ParseResult(name, dlres, res);
 }
 
-size_t multi_write_data(void *ptr, size_t size, size_t nmemb, DownloadData* data)
+static size_t multi_write_data(void *ptr, size_t size, size_t nmemb, DownloadData* data)
 {
 	//LOG_DEBUG("%d %d",size,  nmemb);
 	if (!data->got_ranges) {
@@ -190,7 +216,7 @@ size_t multi_write_data(void *ptr, size_t size, size_t nmemb, DownloadData* data
 	return data->download->file->Write((const char*)ptr, size*nmemb, data->start_piece);
 }
 
-size_t multiHeader(void *ptr, size_t size, size_t nmemb, DownloadData *data)
+static size_t multiHeader(void *ptr, size_t size, size_t nmemb, DownloadData *data)
 {
 	if(data->download->pieces.empty()) { //no chunked transfer, don't check headers
 		LOG_DEBUG("Unchunked transfer!");
@@ -255,19 +281,20 @@ std::vector< unsigned int > CHttpDownloader::verifyAndGetNextPieces(CFile& file,
 	unsigned alreadyDl=0;
 	for(unsigned i=0; i<download->pieces.size(); i++ ) { //find first not downloaded piece
 		showProcess(download, false);
-		if (download->pieces[i].state==IDownload::STATE_FINISHED) {
+		IDownload::piece& p = download->pieces[i];
+		if (p.state == IDownload::STATE_FINISHED) {
 			alreadyDl++;
 			LOG_DEBUG("piece %d marked as downloaded", i);
 			if ( pieces.size() > 0 )
 				break;//Contiguos non-downloaded area finished
 			continue;
-		} else if (download->pieces[i].state==IDownload::STATE_NONE) {
-			if ((download->pieces[i].sha->isSet()) && (!file.IsNewFile())) { //reuse piece, if checksum is fine
+		} else if (p.state==IDownload::STATE_NONE) {
+			if ((p.sha->isSet()) && (!file.IsNewFile())) { //reuse piece, if checksum is fine
 				file.Hash(sha1, i);
 //	LOG("bla %s %s", sha1.toString().c_str(), download.pieces[i].sha->toString().c_str());
 				if (sha1.compare(download->pieces[i].sha)) {
 					LOG_DEBUG("piece %d has already correct checksum, reusing", i);
-					download->pieces[i].state=IDownload::STATE_FINISHED;
+					p.state=IDownload::STATE_FINISHED;
 					showProcess(download, true);
 					alreadyDl++;
 					if ( pieces.size() > 0 )
@@ -285,22 +312,10 @@ std::vector< unsigned int > CHttpDownloader::verifyAndGetNextPieces(CFile& file,
 		download->state=IDownload::STATE_FINISHED;
 		showProcess(download, true);
 	}
-	LOG_DEBUG("Pieces to download: %d\n",pieces.size());
+	LOG_DEBUG("Pieces to download: %d",pieces.size());
 	return pieces;
 }
 
-static int progress_func(DownloadData* data, double total, double done, double, double)
-{
-	data->download->progress = done;
-	if (IDownloader::listener != NULL) {
-		IDownloader::listener(done, total);
-	}
-	if (data->got_ranges) {
-		LOG_PROGRESS(done, total, done >= total);
-	}
-	return 0;
-}
-
 bool CHttpDownloader::setupDownload(DownloadData* piece)
 {
 	std::vector<unsigned int> pieces = verifyAndGetNextPieces(*(piece->download->file), piece->download);
@@ -323,7 +338,7 @@ bool CHttpDownloader::setupDownload(DownloadData* piece)
 	CURL* curle = piece->curlw->GetHandle();
 	piece->mirror=piece->download->getFastestMirror();
 	if (piece->mirror==NULL) {
-		LOG_ERROR("No mirror found");
+		LOG_ERROR("No mirror found for %s", piece->download->name.c_str());
 		return false;
 	}
 
@@ -354,9 +369,11 @@ bool CHttpDownloader::setupDownload(DownloadData* piece)
 
 		//this sets the header If-Modified-Since -> downloads only when remote file is newer than local file
 		const long timestamp = piece->download->file->GetTimestamp();
-		curl_easy_setopt(curle, CURLOPT_TIMECONDITION, CURL_TIMECOND_IFMODSINCE);
-		curl_easy_setopt(curle, CURLOPT_TIMEVALUE, timestamp );
-		curl_easy_setopt(curle, CURLOPT_FILETIME, 1);
+		if ((timestamp >= 0) && (piece->download->hash == nullptr)) { //timestamp known + hash not known -> only dl when changed
+			curl_easy_setopt(curle, CURLOPT_TIMECONDITION, CURL_TIMECOND_IFMODSINCE);
+			curl_easy_setopt(curle, CURLOPT_TIMEVALUE, timestamp );
+			curl_easy_setopt(curle, CURLOPT_FILETIME, 1);
+		}
 	}
 	return true;
 }
@@ -406,20 +423,20 @@ bool CHttpDownloader::processMessages(CURLM* curlm, std::vector <DownloadData*>&
 			}
 			assert(data->download->file!=NULL);
 			assert(data->start_piece< (int)data->download->pieces.size());
-			for ( std::vector<unsigned int>::iterator it = data->pieces.begin(); it != data->pieces.end(); ++it ) {
-				if (data->download->pieces[*it].sha->isSet()) {
-					data->download->file->Hash(sha1, *it);
+			for ( size_t idx = 0; idx < data->pieces.size(); idx++ ) {
+				IDownload::piece& p = data->download->pieces[idx];
+				if (p.sha->isSet()) {
+					data->download->file->Hash(sha1, idx);
 
-					if (sha1.compare(data->download->pieces[*it].sha)) { //piece valid
-
-						data->download->pieces[*it].state=IDownload::STATE_FINISHED;
+					if (sha1.compare(p.sha)) { //piece valid
+						p.state = IDownload::STATE_FINISHED;
 						showProcess(data->download, true);
-						// LOG("piece %d verified!", data->piece);
+						//LOG("piece %d verified!", idx);
 					} else { // piece download broken, mark mirror as broken (for this file)
-						data->download->pieces[*it].state=IDownload::STATE_NONE;
+						p.state = IDownload::STATE_NONE;
 						data->mirror->status=Mirror::STATUS_BROKEN;
 						//FIXME: cleanup curl handle here + process next dl
-						LOG_ERROR("Piece %d is invalid",*it);
+						LOG_ERROR("Piece %d is invalid", idx);
 					}
 				} else {
 					LOG_INFO("sha1 checksum seems to be not set, can't check received piece %d-%d", data->start_piece,data->pieces.size());
@@ -455,6 +472,31 @@ bool CHttpDownloader::processMessages(CURLM* curlm, std::vector <DownloadData*>&
 	return aborted;
 }
 
+static void CleanupDownloads(std::list<IDownload*>& download, std::vector <DownloadData*>& downloads)
+{
+	//close all open files
+	for(IDownload* dl: download) {
+		if (dl->file != nullptr) {
+			dl->file->Close();
+		}
+	}
+	for (size_t i=0; i<downloads.size(); i++) {
+		long timestamp = 0;
+		if ((downloads[i]->curlw != nullptr) && curl_easy_getinfo(downloads[i]->curlw->GetHandle(), CURLINFO_FILETIME, &timestamp) == CURLE_OK) {
+			if (timestamp > 0) {
+				if (downloads[i]->download->state != IDownload::STATE_FINISHED) //decrease local timestamp if download failed to force redownload next time
+					timestamp--;
+				downloads[i]->download->file->SetTimestamp(timestamp);
+			}
+			delete downloads[i]->download->file;
+			downloads[i]->download->file = nullptr;
+		}
+		delete downloads[i];
+	}
+
+	downloads.clear();
+}
+
 bool CHttpDownloader::download(std::list<IDownload*>& download, int max_parallel)
 {
 	std::vector <DownloadData*> downloads;
@@ -486,27 +528,16 @@ bool CHttpDownloader::download(std::list<IDownload*>& download, int max_parallel
 			DownloadData* dlData=new DownloadData();
 			dlData->download=dl;
 			if (!setupDownload(dlData)) { //no piece found (all pieces already downloaded), skip
-				delete dlData;
-				if (dl->state!=IDownload::STATE_FINISHED) {
-					LOG_ERROR("no piece found");
-					return false;
-				}
-			} else {
-				downloads.push_back(dlData);
-				curl_multi_add_handle(curlm, dlData->curlw->GetHandle());
+				LOG_ERROR("Failed to setup download %d/%d", i, count);
+				continue;
 			}
+			downloads.push_back(dlData);
+			curl_multi_add_handle(curlm, dlData->curlw->GetHandle());
 		}
 	}
 	if (downloads.empty()) {
 		LOG_DEBUG("Nothing to download!");
-		//close all open files
-		for(IDownload* dl: download) {
-			if (dl->file != nullptr) {
-				dl->file->Close();
-				delete dl->file;
-				dl->file = nullptr;
-			}
-		}
+		CleanupDownloads(download, downloads);
 		return true;
 	}
 
@@ -543,49 +574,13 @@ bool CHttpDownloader::download(std::list<IDownload*>& download, int max_parallel
 		//sleep for one sec / until something happened
 		select(count+1, &rSet, &wSet, &eSet, &t);
 	}
-	if (!aborted) { // if download didn't fail, get file size reported in http-header
-		double size=-1;
-		for (unsigned i=0; i<downloads.size(); i++) {
-			if (downloads[i]->curlw == nullptr)
-				continue;
-			double tmp;
-			curl_easy_getinfo(downloads[i]->curlw->GetHandle(), CURLINFO_CONTENT_LENGTH_DOWNLOAD, &tmp);
-			if (tmp>size) {
-				size=tmp;
-			}
-		}
-		//set download size if isn't set and we have a valid number
-//		if ((size>0) && (download->size<0)) {
-//  		download->size = size;
-//		}
 
-	}
-//	showProcess(download, file);
 	LOG("\n");
 
 	if (!aborted) {
 		LOG_DEBUG("download complete");
 	}
-
-	//close all open files
-	for(IDownload* dl: download) {
-		if (dl->file != nullptr) {
-			dl->file->Close();
-		}
-	}
-	for (size_t i=0; i<downloads.size(); i++) {
-		long timestamp;
-		if ((downloads[i]->curlw != nullptr) && curl_easy_getinfo(downloads[i]->curlw->GetHandle(), CURLINFO_FILETIME, &timestamp) == CURLE_OK) {
-			if (downloads[i]->download->state != IDownload::STATE_FINISHED) //decrease local timestamp if download failed to force redownload next time
-				timestamp--;
-			downloads[i]->download->file->SetTimestamp(timestamp);
-			delete downloads[i]->download->file;
-			downloads[i]->download->file = nullptr;
-		}
-		delete downloads[i];
-	}
-
-	downloads.clear();
+	CleanupDownloads(download, downloads);
 	curl_multi_cleanup(curlm);
 	return !aborted;
 }
